package sequoia.path.alignment;

import sequoia.path.graph.Node;

public class CellUnitKey
{
    static final String UNDEF_KEY = "--";

    static final String GAP_KEY = "-";

    static final String EMPTY_KEY = "";

    public final Node nA;

    public final Node nB;

    public final String key;

    public CellUnitKey()
    {
        this.nA = Node.ANY_NODE;
        this.nB = Node.ANY_NODE;
        this.key = EMPTY_KEY;
    }

    public CellUnitKey(String k)
    {
        this.nA = Node.ANY_NODE;
        this.nB = Node.ANY_NODE;
        this.key = k;
    }

    public CellUnitKey(Node n1, Node n2)
    {
        this.nA = n1;
        this.nB = n2;
        if (this.nA == null) {
            if (this.nB == null) {
                this.key = EMPTY_KEY;
            } else {
                this.key = GAP_KEY + this.nB.getSymbol();
            }
        } else if (this.nB == null) {
            this.key = this.nA.getSymbol() + GAP_KEY;
        } else {
            this.key = this.nA.getSymbol() + "" + this.nB.getSymbol();
        }
    }

    @Override
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((this.key == null) ? 0 : this.key.hashCode());
        result = PRIME * result + ((this.nA == null) ? 0 : this.nA.hashCode());
        result = PRIME * result + ((this.nB == null) ? 0 : this.nB.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj) {
            return true;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final CellUnitKey other = (CellUnitKey) obj;
        if (this.key == null) {
            if (other.key != null) {
                return false;
            }
        } else if (!this.key.equals(other.key)) {
            return false;
        }
        if (this.nA == null) {
            if (other.nA != null) {
                return false;
            }
        } else if (!this.nA.equals(other.nA)) {
            return false;
        }
        if (this.nB == null) {
            if (other.nB != null) {
                return false;
            }
        } else if (!this.nB.equals(other.nB)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString()
    {
        return "{{" + this.nA.toShortString() + "," + this.nB.toShortString() + "}}";
    }
}
