package sequoia.path.alignment;

import sequoia.path.graph.Node;
import sequoia.path.graph.BackTranslationGraph;
import sequoia.path.util.Codon;

public class GlobalDPFSCGapAlignment extends GlobalDPAlignment
{
    int performedFrameshifts = 0;

    /**
     * Constructor
     * 
     * @param A
     * @param B
     * @param d
     * @param e
     * @param m
     * @param x
     * @param codonSkip
     * @param codonSkipExtension
     * @param fsNo
     */
    public GlobalDPFSCGapAlignment(BackTranslationGraph A, BackTranslationGraph B, int d, int e, int m, int x,
        int codonSkip, int codonSkipExtension, int fsNo)
    {
        super(A, B, d, e, m, x);
        this.codonSkip = codonSkip;
        this.codonSkipExtension = codonSkipExtension;
        this.allowedFrameshifts = fsNo;

    }

    /**
     * Compute the score of a given cell
     */
    @Override
    protected void computeBestPartialScore(int i, int j, CellUnitKey k)
    {

        Node aNodes[] = this.A.getPrecNodes(i - 1, k.nA), bNodes[] = this.B.getPrecNodes(j - 1, k.nB);

        CellUnitInfo c = this.alignmentMatrix[i][j].get(k);
        c.score = Integer.MIN_VALUE;
        // c.setMyStart(new Coord(i, j, k));

        partialScoreOptionMatch(c, aNodes, bNodes, i, j, k);
        partialScoreOptionHFrameShift(c, aNodes, i, j, k);
        partialScoreOptionVFrameShift(c, bNodes, i, j, k);

        if (this.A.isCodonskipPosition(i)) {
            // System.out.println("CSP " + i);
            aNodes = this.A.getNodes(i - Codon.CODON_LEN - 1);
            partialScoreOptionHCodonGap(c, aNodes, i, j, k);
        }
        if (this.B.isCodonskipPosition(j)) {
            // System.out.println("CSP " + j);
            bNodes = this.B.getNodes(j - Codon.CODON_LEN - 1);
            partialScoreOptionVCodonGap(c, bNodes, i, j, k);
        }
        // System.out.println("[" + i + " " + j + " " + k + "] " + c.score + " " + c.prevCell.Di + " " + c.prevCell.Dj);
    }

    /**
     * Obtain the end of the best alignment (for traceback)
     */
    @Override
    public CellUnitKey getBestUnitKey(CellInfo cell)
    {
        this.bestScore = Integer.MIN_VALUE;
        CellUnitKey bestKey = null;
        CellUnitKey[] poss = cell.getKeys();
        for (int i = 0; i < poss.length; ++i) {
            if (cell.get(poss[i]).score > this.bestScore) {
                this.bestScore = cell.get(poss[i]).score;
                // this.performedFrameshifts = cell.get(poss[i]).frameshiftCount;
                // System.out.println("############################ " + performedFrameshifts);
                bestKey = poss[i];
            } else if (cell.get(poss[i]).score == this.bestScore
                && cell.get(poss[i]).getPrevCell().getDi() == cell.get(poss[i]).getPrevCell().getDj()) {
                bestKey = poss[i];
            }
        }
        // System.out.println("++++++++++++++++++++ " + performedFrameshifts);
        return bestKey;
    }
}
