package sequoia.path.alignment;

import sequoia.path.graph.Node;
import sequoia.path.graph.BackTranslationGraph;
import sequoia.path.util.Codon;

public class OverlapDPFSCGapAlignment extends OverlapDPAlignment
{

    public OverlapDPFSCGapAlignment(BackTranslationGraph A, BackTranslationGraph B, int d, int e, int m, int x,
        int codonSkip, int fsNo)
    {
        super(A, B, d, e, m, x);
        this.codonSkip = codonSkip;
        this.codonSkipExtension = codonSkip;
        allowedFrameshifts = fsNo;

    }
    
    public OverlapDPFSCGapAlignment(BackTranslationGraph A, BackTranslationGraph B, int d, int e, int m, int x,
            int codonSkip, int codonSkipExtension, int fsNo)
        {
            super(A, B, d, e, m, x);
            this.codonSkip = codonSkip;
            this.codonSkipExtension = codonSkipExtension;
            allowedFrameshifts = fsNo;

        }

    // ************************************************************************
    // C O M P U T E T H E S C O R E O F E A C H C E L L
    @Override
    protected void computeBestPartialScore(int i, int j, CellUnitKey k)
    {

        Node aNodes[] = A.getPrecNodes(i - 1, k.nA), bNodes[] = B.getPrecNodes(j - 1, k.nB);

        CellUnitInfo c = alignmentMatrix[i][j].get(k);
        c.score = Double.MIN_VALUE;
        //c.setMyStart(new Coord(i, j, k));
        //c.frameshiftCount = (int) Math.signum(Math.abs(A.getPhase(i) - B.getPhase(j)));

        partialScoreOptionMatch(c, aNodes, bNodes, i, j, k);
        partialScoreOptionHFrameShift(c, aNodes, i, j, k);
        partialScoreOptionVFrameShift(c, bNodes, i, j, k);

        if (A.isCodonskipPosition(i)) {
            // System.out.println("CSP " + i);
            aNodes = A.getNodes(i - Codon.CODON_LEN - 1);
            partialScoreOptionHCodonGap(c, aNodes, i, j, k);
        }
        if (B.isCodonskipPosition(j)) {
            // System.out.println("CSP " + j);
            bNodes = B.getNodes(j - Codon.CODON_LEN - 1);
            partialScoreOptionVCodonGap(c, bNodes, i, j, k);
        }
        // System.out.println("[" + i + " " + j + " " + k + "] " + c.score + " " + c.prevCell.Di + " " + c.prevCell.Dj);
    }

}
