package sequoia.path.alignment;

import sequoia.path.graph.Node;
import sequoia.path.scores.ScoreMatrix;
import sequoia.path.util.GenomicSequenceUtils;
import sequoia.path.util.ProteinSeqUtils;

public class SubstitutionScoreOracle
{
    protected final static int UNDEFINED = -1;
    protected final double MATCH_BONUS = 0;//1.0 / (64*64);
    protected final double TRANSITION_BONUS = 0;//1.0 / (64*64*2);
    protected final double TRANSVERTION_BONUS = -0;

    ScoreMatrix sm = null;

    double m, x, tx;

    public SubstitutionScoreOracle(ScoreMatrix M, double match, double mismatch, double tmismatch)
    {
        sm = M;
        m = match;
        x = mismatch;
        tx = tmismatch;
    }

    public double getScore(Node a, Node b)
    {
    	boolean rcA = ProteinSeqUtils.isRC(a.getAA());
    	char nA = a.getSymbol();; //rcA ? GenomicSequenceUtils.getComplementary(a.getSymbol()) : a.getSymbol();
    	char aaA = ProteinSeqUtils.toStd(a.getAA());
    	int pA = a.getPlace(); //rcA ? Codon.CODON_LEN - 1 - a.getPlace() : a.getPlace();
    	
    	boolean rcB = ProteinSeqUtils.isRC(b.getAA());
    	char nB = b.getSymbol(); //rcB ? GenomicSequenceUtils.getComplementary(b.getSymbol()) : b.getSymbol();
    	char aaB = ProteinSeqUtils.toStd(b.getAA());
    	int pB = b.getPlace(); //rcB ? Codon.CODON_LEN - 1 - b.getPlace() : b.getPlace();

        try {
        	return sm.getScore(nA, aaA, pA, rcA, nB, aaB, pB, rcB) + 
        	(nA == nB ? MATCH_BONUS : (GenomicSequenceUtils.isTransitionMutation(nA, nB))? TRANSITION_BONUS : TRANSVERTION_BONUS);
        } catch (Exception ex) {
        //	System.out.println("Not found: " +  nA + " " + aaA + " " + pA + " " + rcA + "   " + nB + " " + aaB + " " + pB + " " + rcB);
            return getScore(nA, nB);
        }
    }

    public String getNPair(Node a, Node b)
    {
        try {
            return sm.getNPair(a.getSymbol(), a.getAA(), a.getPlace(), ProteinSeqUtils.isRC(a.getAA()), b.getSymbol(),
                b.getAA(), b.getPlace(), ProteinSeqUtils.isRC(b.getAA()));

        } catch (Exception ex) {
            return getNPair(a.getSymbol(), b.getSymbol());
        }
    }

    protected double getScore(char a, char b)
    {
        if (GenomicSequenceUtils.isMatch(a, b)) {
            return m;
        } else if (GenomicSequenceUtils.isTransitionMutationExt(a, b)) {
            return tx;
        }
        return x;
    }

    protected String getNPair(char a, char b)
    {
        return a + "" + b;
    }

    public ScoreMatrix getScoreMatrix()
    {
        return sm;
    }

}
