package sequoia.path.alignment.estimation;

import java.util.HashMap;

import sequoia.path.alignment.CellUnitInfo;
import sequoia.path.alignment.CellUnitKey;
import sequoia.path.alignment.Coord;
import sequoia.path.alignment.estimation.ScoreEvaluationParameters.ReadingFrameDifference;
import sequoia.path.alignment.spaceecon.AlignmentPoints;
import sequoia.path.alignment.spaceecon.ScoreList;
import sequoia.path.alignment.spaceecon.SpaceEconomicLocalDPFSCAligner;
import sequoia.path.graph.Node;
import sequoia.path.graph.BackTranslationGraph;
import sequoia.path.util.Codon;

public class ScoreEvaluationParametersAlignment extends
		SpaceEconomicLocalDPFSCAligner {
	
	/** */
	protected AlignmentScoreSignificanceEstimation estimator;
	protected ReadingFrameDifference readingFrameDifference;
    
	public ReadingFrameDifference getReadingFrameDifference() {
		return readingFrameDifference;
	}
	public void setReadingFrameDifference(ReadingFrameDifference df) {
		this.readingFrameDifference = df;
	}
	// ************************************************************************
    // C O N S T R U C T O R S
	public ScoreEvaluationParametersAlignment(BackTranslationGraph A,
			BackTranslationGraph B, int d, int e, int m, int x, int codonSkip,
			int codonSkipExtend, int fsNo, double acceptedScore) {
		super(A, B, d, e, m, x, codonSkip, codonSkipExtend, fsNo, acceptedScore);
		filter = new NullFilter();
		readingFrameDifference = ReadingFrameDifference.valueOf(this.isRC());
	}
	public ScoreEvaluationParametersAlignment(BackTranslationGraph A,
			BackTranslationGraph B, int d, int e, int m, int x, int codonSkip,
			int codonSkipExtend, int fsNo, double acceptedScore,
			ReadingFrameDifference df) {
		super(A, B, d, e, m, x, codonSkip, codonSkipExtend, fsNo, acceptedScore);
		filter = new NullFilter();
		this.readingFrameDifference = df;
		if (!flexiblePath()) {
			this.d = Integer.MIN_VALUE >> 1;
			this.e = Integer.MIN_VALUE >> 1;
		}
	}

	// ************************************************************************
	private boolean flexiblePath() {
		return this.readingFrameDifference == ReadingFrameDifference.FW || readingFrameDifference == ReadingFrameDifference.RC;
	}
    /**
     * Override match to avoid starting from points that do not comply with the imposed reading frame differences
     */
    protected void partialScoreOptionMatch(CellUnitInfo c, Node aNodes[], Node bNodes[], int i, int j,
        CellUnitKey k)
    {
    	int realI = this.windowToMartixIdx(i);
    	int crtRFD = 0;
		if (this.isRC()) {
			crtRFD = -((this.A.getPhase(realI) - this.B.getPhase(j) + Codon.CODON_LEN) % Codon.CODON_LEN);
			if (crtRFD == 0) {
				crtRFD = -Codon.CODON_LEN;
			}
		} else {
			crtRFD = ((this.A.getPhase(realI) - this.B.getPhase(j) + Codon.CODON_LEN) % Codon.CODON_LEN);
		}
    	if (flexiblePath() || crtRFD == readingFrameDifference.value) {
    		super.partialScoreOptionMatch(c, aNodes, bNodes, i, j, k);
    	} 
    }
	
	public void setEstimator(AlignmentScoreSignificanceEstimation e) {
		this.estimator = e;
		this.list = new ScoreList(e.getMinScore(this.isRC()));
	}
	
    public HashMap<Coord, AlignmentPoints> getIslands() {
        return this.list.getStarts();
    }
    
    public void acknowledgeScore(int i, int j, CellUnitKey k)
    {
        this.list.updateAll(i, j, k, this);
    }
}
