package sequoia.path.alignment.format;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import sequoia.path.alignment.Alignment;
import sequoia.path.alignment.AlignmentTracer;
import sequoia.path.alignment.FrameshiftFragment;

public enum XMLFormatter implements AlignmentFormatter{
	
	instance;
	

    private final static String AA_CHAR_PATTERN = "[a-zA-Z&&[^oubxzOUBXZ]]";
    private final static int CHARS_PER_ROW = 90;
	
	private String formatAA(String str, char prevAA, char nextAA) {
        String result = str.replaceAll("(?:\\[)(\\s*)(" + AA_CHAR_PATTERN + ")(\\s*)(?:\\])", "<span class='$2'> $1$2$3 </span>");
        result = result.replaceAll("^(\\s*)(" + AA_CHAR_PATTERN + ")(\\s*)(?:\\])", "<span class='$2'>$1$2$3 </span>");
        result = result.replaceAll("^(\\s*)(?:\\])", "<span class='" + prevAA + "'>$1 </span>");
        result = result.replaceAll("(?:\\[)(\\s*)(" + AA_CHAR_PATTERN + ")(\\s*)$", "<span class='$2'> $1$2$3</span>");
        result = result.replaceAll("(?:\\[)(\\s*)$", "<span class='" + nextAA + "'> $1</span>");
    	return result;
    }
    
    private String formatBase(String str) {
        String result = str.replaceAll("(?<![-^])(?:---)*+-{1,2}(?![-$])", "<span class='fs'>$0</span>");
        return result;
    }
    
    private String formatMatch(String strM, String strS) {
    	StringBuilder str = new StringBuilder();
    	String type;
    	for (int i = 0; i < strM.length(); ++i) {
    		switch (strS.charAt(i)){
    		case AlignmentTracer.POSITIVE_SCORE:
    			type = "s-p";
    			break;
    		case AlignmentTracer.SMALL_POSITIVE_SCORE:
    			type = "s-ps";
    			break;
    		case AlignmentTracer.ZERO_SCORE:
     			type = "s-z";
     			break;
    		case AlignmentTracer.SMALL_NEGATIVE_SCORE:
    			type = "s-ns";
    			break;
    		case AlignmentTracer.NEGATIVE_SCORE:
    			type = "s-n";
    			break;
    		default:
    			type="";
    			break;
    		}
     		
    		str.append("<span class='");
    		str.append(type);
    		str.append("'>");
    		str.append(strM.charAt(i));
    		str.append("</span>");
    	}
    	return str.toString();
    }
	
	protected String colorAlignment(Alignment alignment) {
        StringBuilder str = new StringBuilder();
        AlignmentTracer t = alignment.getTracer();
        String strA = t.getNucleotideSequenceA();
        String strB = t.getNucleotideSequenceB();
        String strpA = t.getAASequenceA();
        String strpB = t.getAASequenceB();
        String matcher = t.getMatchSequence();
        String strS = t.getScoreSignSequence();
        
        String prevALine = "X", prevBLine = "X", currentALine, currentBLine, nextALine, nextBLine;
        int end = strA.length(), segmentStart = 0, segmentEnd = Math.min(CHARS_PER_ROW, end);
        String a = strpA + "X", b = strpB + "X";

        nextALine = a.substring(segmentEnd);
        nextBLine = b.substring(segmentEnd);
        currentALine = a.substring(segmentStart, segmentEnd);
        currentBLine = b.substring(segmentStart, segmentEnd);
        Pattern prevCharPattern = Pattern.compile(".*(" + AA_CHAR_PATTERN + ")\\s*$");
        Pattern nextCharPattern = Pattern.compile("^\\s*(" + AA_CHAR_PATTERN + ").*");
        Matcher prevCharMatcher = prevCharPattern.matcher("");
        Matcher nextCharMatcher = nextCharPattern.matcher("");;
        char prevChar, nextChar;

        while (segmentEnd <= end) {
            if (alignment.A.isBackTranslation()) {
                prevCharMatcher.reset(prevALine);
                if (prevCharMatcher.matches()) {
                    prevChar = prevCharMatcher.group(1).charAt(0);
                } else {
                    prevChar = 'X';
                }
                nextCharMatcher.reset(nextALine);
                if (nextCharMatcher.matches()) {
                    nextChar = nextCharMatcher.group(1).charAt(0);
                } else {
                    nextChar = 'X';
                }
                str.append(formatAA(currentALine, prevChar, nextChar));
                str.append('\n');
            }
            str.append(formatBase(strA.substring(segmentStart, segmentEnd)));
            str.append('\n');
            str.append(formatMatch(matcher.substring(segmentStart, segmentEnd), strS.substring(segmentStart, segmentEnd)));
            str.append('\n');
            str.append(formatBase(strB.substring(segmentStart, segmentEnd)));
            str.append('\n');
            if (alignment.B.isBackTranslation()) {
                prevCharMatcher.reset(prevBLine);
                if (prevCharMatcher.matches()) {
                    prevChar = prevCharMatcher.group(1).charAt(0);
                } else {
                    prevChar = 'X';
                }
                nextCharMatcher.reset(nextBLine);
                if (nextCharMatcher.matches()) {
                    nextChar = nextCharMatcher.group(1).charAt(0);
                } else {
                    nextChar = 'X';
                }
                str.append(formatAA(currentBLine, prevChar, nextChar));
                str.append('\n');
            }
            segmentStart += CHARS_PER_ROW;
            segmentEnd += CHARS_PER_ROW;
            if (segmentStart >= end) {
                break;
            }
            if (segmentEnd < end) {
                nextALine = nextALine.substring(CHARS_PER_ROW);
                nextBLine = nextBLine.substring(CHARS_PER_ROW);
            } else {
                segmentEnd = end;
                nextALine = "X";
                nextBLine = "X";
            }
            prevALine = currentALine;
            prevBLine = currentBLine;
            currentALine = a.substring(segmentStart, segmentEnd);
            currentBLine = b.substring(segmentStart, segmentEnd);
            str.append('\n');
            str.append('\n');
            str.append('\n');
        }
        return str.toString();
    }
	/**
	 * Exports an alignment in XML format. To be called only after both align() and traceback() were performed for that alignment object.
	 * @return a string containing the XML content
	 */
	public String format(Alignment alignment) {
		 StringBuilder str = new StringBuilder();
	        str.append("  <alignmentInfo>\n");;
	        str.append("    <sequences>\n");
	        str.append("      <sequence>\n");
	        str.append(alignment.getPseqA().toFasta());
	        str.append("      </sequence>\n");
	        str.append("      <sequence>\n");
	        str.append(alignment.getPseqB().toFasta());
	        str.append("      </sequence>\n");
	        str.append("    </sequences>\n");
	        str.append("    <alignment>\n");
	        str.append(colorAlignment(alignment) + "\n");
	        str.append("    </alignment>\n");
	        str.append("    <score>");
	        str.append(String.format("%.2f", alignment.getScore()));
	        str.append("</score>\n");
	        str.append("    <pvalue>");
	        str.append(alignment.getScoreSignificance());
	        str.append("</pvalue>\n");
	        str.append("    <fragments>\n");
	        for (FrameshiftFragment f : alignment.getFragments()) {
	            str.append("      <fragment>\n");
	            str.append("        <startA>");
	        	str.append(f.getFrom().getDi());
	        	str.append("</startA>\n");
	            str.append("        <startB>");
	        	str.append(f.getFrom().getDj());
	        	str.append("</startB>\n");
	            str.append("        <endA>");
	        	str.append(f.getTo().getDi());
	        	str.append("</endA>\n");
	            str.append("        <endB>");
	        	str.append(f.getTo().getDj());
	        	str.append("</endB>\n");
	            str.append("        <phase>");
	            str.append(f.getPhaseDifference(alignment));
	            str.append("</phase>\n");
	            str.append("        <score>");
	        	str.append(String.format("%.2f", f.getFragmentScore()));
	        	str.append("</score>\n");
	            str.append("        <fpvalue>");
	        	str.append(alignment.getScoreSignificance(f));
	        	str.append("</fpvalue>\n");
	        	str.append("      </fragment>\n");
	        }
	        str.append("    </fragments>\n");
	        str.append("  </alignmentInfo>\n");
	        return str.toString();
	}
}
