package sequoia.path.alignment.spaceecon;

import sequoia.path.alignment.CellUnitKey;
import sequoia.path.alignment.Coord;
import sequoia.path.graph.Node;

public class ScanWindow
{
    // Seqence A labels the columns (so it is horizontal); it is traversed by index i (or c...)
    // Seqence B labels the rowq (so it is vertical ); it is traversed by index j
    ScanWindowCellInfo[][] window;

    int shift = 0;

    final int hSteps;

    final SpaceEconomicAligner onAlignment;

    // ************************************************************************
    // C O N S T R U C T O R
    public ScanWindow(SpaceEconomicAligner a, int columns)
    {
        this.onAlignment = a;
        int vSeqSize = a.B.size();
        int hSeqSize = a.A.size();
        this.window = new ScanWindowCellInfo[columns][];
        this.hSteps = hSeqSize - columns + 1;
        for (int i = 0; i < this.window.length; ++i) {
            this.window[i] = new ScanWindowCellInfo[vSeqSize + 1];
            for (int j = 0; j < this.window[i].length; ++j) {
                this.window[i][j] = new ScanWindowCellInfo();
            }
        }
        initBorderColumnData(0);
        for (int i = 1; i < this.window.length; ++i) {
            resetColumnData(i);
        }
    }

    // ************************************************************************
    // R E S E T D A T A
    protected void resetColumnData(int c)
    {
        // compute the actual position in the horizontal sequence this column referes to
        // NB! the 'horizontal' sequence is the first one (A)
        int fullMatIdx = windowToMatrixIdx(c);
        int seqPosition = fullMatIdx - 1;
        Node[] colNodes = this.onAlignment.A.getNodes(seqPosition);
        if (colNodes == null) {
            colNodes = new Node[1];
            colNodes[0] = Node.ANY_NODE;
        }
        this.window[c][0].clear();
        for (int k = 0; k < colNodes.length; ++k) {
            CellUnitKey key = new CellUnitKey(colNodes[k], Node.ANY_NODE);
            this.window[c][0].put(key, new ScanWindowCellUnitInfo(this.onAlignment.getBorderInitValue(fullMatIdx),
                new Coord(fullMatIdx, 0, key)));
        }
        for (int j = 1; j < this.window[c].length; ++j) {
            this.window[c][j].clear();
            Node[] rowNodes = this.onAlignment.B.getNodes(j - 1);
            for (int k = 0; k < colNodes.length; ++k) {
                for (int h = 0; h < rowNodes.length; ++h) {
                    CellUnitKey key = new CellUnitKey(colNodes[k], rowNodes[h]);
                    this.window[c][j].put(key, new ScanWindowCellUnitInfo(this.onAlignment.getCellUnitInitValue(),
                        new Coord(fullMatIdx, j, key)));
                }
            }
        }

    }

    protected void initBorderColumnData(int c)
    {
        // compute the actual position in the horizontal sequence this column referes to
        // NB! the 'horizontal' sequence is the first one (A)

        Node[] colNodes = new Node[1];
        colNodes[0] = Node.ANY_NODE;

        this.window[c][0].clear();
        for (int k = 0; k < colNodes.length; ++k) {
            CellUnitKey key = new CellUnitKey(colNodes[k], Node.ANY_NODE);
            this.window[c][0].put(key, new ScanWindowCellUnitInfo(this.onAlignment.getBorderInitValue(c), new Coord(c,
                0, key)));
        }
        for (int j = 1; j < this.window[c].length; ++j) {
            this.window[c][j].clear();
            Node[] rowNodes = this.onAlignment.B.getNodes(j - 1);
            for (int k = 0; k < colNodes.length; ++k) {
                for (int h = 0; h < rowNodes.length; ++h) {
                    CellUnitKey key = new CellUnitKey(colNodes[k], rowNodes[h]);
                    this.window[c][j].put(key, new ScanWindowCellUnitInfo(this.onAlignment.getBorderInitValue(j),
                        new Coord(c, j, key)));
                }
            }
        }

    }

    // ************************************************************************
    // S H I F T W I N D O W
    public boolean shiftWindow()
    {
        if (this.shift == this.hSteps) {
            return false;
        }
        ++this.shift;
        ScanWindowCellInfo[] tmp = this.window[0];
        for (int i = 1; i < this.window.length; ++i) {
            this.window[i - 1] = this.window[i];
        }
        this.window[this.window.length - 1] = tmp;
        resetColumnData(this.window.length - 1);
        return true;
    }

    // ************************************************************************
    // A C C E S S F U N C T I O N S T O S O M E D A T A
    int size()
    {
        return this.window.length;
    }

    ScanWindowCellInfo[] getColumn(int idx)
    {
        return this.window[idx];
    }

    public ScanWindowCellUnitInfo getCellUnit(Coord co)
    {
        // do we have it in the window?
        if (hasColumn(co.getDi())) {
            return this.window[matrixToWindowIdx(co.getDi())][co.getDj()].get(co.getFullKey());
        }
        return null;
    }

    public ScanWindowCellUnitInfo getCellUnit(Coord co, boolean convert)
    {
        if (convert) {
            return getCellUnit(co);
        }
        return this.window[co.getDi()][co.getDj()].get(co.getFullKey());
    }

    public ScanWindowCellUnitInfo getCellUnit(int i, int j, CellUnitKey k)
    {
        return this.window[i][j].get(k);
    }

    public ScanWindowCellInfo getCell(int i, int j)
    {
        return this.window[i][j];
    }

    public ScanWindowCellUnitInfo getCellUnit(int i, int j, Node a, Node b)
    {
        return this.window[i][j].get(a, b);
    }

    public boolean hasColumn(int i)
    {
        int windowIdx = i - this.shift;
        return (windowIdx >= 0) && (windowIdx < this.window.length);
    }

    // ************************************************************************
    // I N D E X C O N V E R S I O N S
    protected int matrixToWindowIdx(int colIdx)
    {
        return colIdx - this.shift;
    }

    protected int windowToMatrixIdx(int colIdx)
    {
        return colIdx + this.shift;
    }

    protected Coord matrixToWindowCoord(Coord co)
    {
        return new Coord(matrixToWindowIdx(co.getDi()), co.getDj(), co.getFullKey());
    }

    protected Coord windowToMatrixCoord(Coord co)
    {
        return new Coord(windowToMatrixIdx(co.getDi()), co.getDj(), co.getFullKey());
    }

    // ************************************************************************
    // T O S T R I N G
    @Override
    public String toString()
    {
        StringBuilder str = new StringBuilder();
        for (int i = 0; i < this.window.length; ++i) {
            str.append(toString(i));
        }
        return str.toString();
    }

    public String toString(int c)
    {
        StringBuilder str = new StringBuilder();
        str.append("__________________________________________\n");
        int col = windowToMatrixIdx(c);
        for (int j = 0; j < this.window[c].length; ++j) {
            str.append("[" + col + ", " + j + "] ");
            str.append(this.window[c][j]);
            str.append('\n');
        }
        str.append("------------------------------------------\n");
        return str.toString();
    }
}
