package sequoia.path.alignment.spaceecon;

import java.util.ArrayList;
import java.util.HashMap;

import sequoia.path.alignment.CellUnitKey;
import sequoia.path.alignment.Coord;

public class ScoreList
{
    public HashMap<Coord, AlignmentPoints> getStarts() {
		return starts;
	}

	public void setStarts(HashMap<Coord, AlignmentPoints> starts) {
		this.starts = starts;
	}

	public void setScore(double score) {
		this.score = score;
	}

	protected double score;

    final boolean rememberAll;

    ArrayList<AlignmentPoints> list = new ArrayList<AlignmentPoints>();

    HashMap<Coord, AlignmentPoints> starts = new HashMap<Coord, AlignmentPoints>();

    public ScoreList()
    {
        this(0, false);
    }

    public ScoreList(double minScore)
    {
        this(minScore, false);
    }

    public ScoreList(double minScore, boolean memAll)
    {
        score = minScore;
        rememberAll = memAll;
    }

    public boolean update(Coord co, SpaceEconomicLocalDPFSCAligner a)
    {
        ScanWindowCellUnitInfo c = (ScanWindowCellUnitInfo) a.getCellUnit(co);
        if (c.score < score) {
            return false;
        }
        if (c.score > score) {
            score = c.score;
            list.clear();
        }
        if (rememberAll || c.score > score) {
            list.add(new AlignmentPoints(c.getMyStart(), co, a.getMilestonesFor(co), c.score));
            return true;
        }
        return false;
    }

    public boolean updateAll(Coord co, SpaceEconomicLocalDPFSCAligner a)
    {
        ScanWindowCellUnitInfo c = (ScanWindowCellUnitInfo) a.getCellUnit(co);
        if (c.score < score) {
            return false;
        }

        AlignmentPoints pts = starts.get(c.getMyStart()), npts =
            new AlignmentPoints(c.getMyStart(), co, a.getMilestonesFor(co), c.score);

        if (pts != null && pts.score < c.score) {
            starts.put(c.getMyStart(), npts);
            list.remove(pts);
        }

        list.add(npts);
        return true;
    }

    public boolean update(int i, int j, CellUnitKey k, SpaceEconomicAligner a)
    {
        ScanWindowCellUnitInfo c = (ScanWindowCellUnitInfo) a.getCellUnit(i, j, k);
        if (c.score < score) {
            return false;
        }
        boolean addThis = rememberAll || c.score > score;
        if (c.score > score) {
            score = c.score;
            a.setScore(c.score);
            list.clear();
        }
        if (addThis) {
            Coord co = new Coord(i, j, k);
            AlignmentPoints p = new AlignmentPoints(c.getMyStart(), a.win.windowToMatrixCoord(co), a.getMilestonesFor(co), c.score);
            if (p.points.size() >= 2 && p.points.get(p.points.size() - 1).equals(p.points.get(p.points.size() - 2))) {
            	p.points.remove(p.points.size() - 1);
            }
            list.add(p);
            return true;
        }
        return false;
    }

    public boolean updateAll(int i, int j, CellUnitKey k, SpaceEconomicFSCAligner a)
    {
        ScanWindowCellUnitInfo c = (ScanWindowCellUnitInfo) a.getCellUnit(i, j, k);
        if (c.score < score) {
            return false;
        }

        Coord co = new Coord(i, j, k);
        AlignmentPoints pts = starts.get(c.getMyStart()), npts =
            new AlignmentPoints(c.getMyStart(), a.win.windowToMatrixCoord(co), a.getMilestonesFor(co), c.score);

        if (pts != null && pts.score < c.score) {
            list.remove(pts);
        }
        if (pts == null || pts.score < c.score) {
            starts.put(c.getMyStart(), npts);
            list.add(npts);
        }
        if (a.getScore() < c.score) {
            a.setScore(c.score);
        }
        // if (starts.get(key))
        return true;
    }

    public void putGlobalAlignmentFor(SpaceEconomicAligner al)
    {
        list.add(new AlignmentPoints(new Coord(0, 0, new CellUnitKey()), new Coord(al.A.size(), al.B.size(),
            new CellUnitKey()), null, 0));
    }

    public double getScore()
    {
        return score;
    }

    public int size()
    {
        return list.size();
    }

    public AlignmentPoints get(int i)
    {
        return list.get(i);
    }

    @Override
    public String toString()
    {
        StringBuilder str = new StringBuilder();
        str.append("Best local alignment score: ");
        str.append(score);
        str.append('\n');
        for (int i = 0; i < list.size(); ++i) {
            str.append(i);
            str.append(":");
            str.append(list.get(i));
            str.append('\n');
        }
        return str.toString();
    }
}
