package sequoia.path.alignment.spaceecon;

import sequoia.path.alignment.Alignment;
import sequoia.path.alignment.CellUnitKey;
import sequoia.path.alignment.GlobalDPFSCGapAlignment;
import sequoia.path.graph.Node;
import sequoia.path.graph.BackTranslationGraph;
import sequoia.path.util.Codon;

abstract public class SpaceEconomicFSCAligner extends SpaceEconomicAligner
{
    // Sequence A labels the columns (so it is horizontal); it is traversed by index i (or c...)
    // Sequence B labels the rows (so it is vertical ); it is traversed by index j
    boolean largeGaps = true;

    // ************************************************************************
    // C O N S T R U C T O R S
    public SpaceEconomicFSCAligner(BackTranslationGraph a, BackTranslationGraph b, int scoreAtLeast)
    {
        super(a, b, scoreAtLeast);
    }

    public SpaceEconomicFSCAligner(BackTranslationGraph A, BackTranslationGraph B, int d, int e, int m, int x,
        int codonSkip, int codonSkipExtension, int fsNo, double acceptedScore)
    {
        super(A, B, d, e, m, x, codonSkip, codonSkipExtension, fsNo, acceptedScore);
    }

    // ************************************************************************
    /**
     * compute the score in each cell
     */
    @Override
    protected void computeBestPartialScore(int i, int j, CellUnitKey k)
    {
        int seqI = this.win.windowToMatrixIdx(i);
        Node aNodes[] = this.A.getPrecNodes(seqI - 1, k.nA), bNodes[] = this.B.getPrecNodes(j - 1, k.nB);

        ScanWindowCellUnitInfo c = (ScanWindowCellUnitInfo) getCellUnit(i, j, k);
        c.score = this.getCellUnitInitValue();
        c.frameshiftCount = this.A.getPhase(i) != this.B.getPhase(j) ? 1 : 0;
        initCellUnitStartField(c, i, j, k);

        partialScoreOptionMatch(c, aNodes, bNodes, i, j, k);
        partialScoreOptionHFrameShift(c, aNodes, i, j, k);
        partialScoreOptionVFrameShift(c, bNodes, i, j, k);

        if (this.A.isCodonskipPosition(seqI)) {
            aNodes = this.A.getNodes(seqI - Codon.CODON_LEN - 1);
            partialScoreOptionHCodonGap(c, aNodes, i, j, k);
        }
        if (this.B.isCodonskipPosition(j)) {
            // System.out.println("CSP " + j);
            bNodes = this.B.getNodes(j - Codon.CODON_LEN - 1);
            partialScoreOptionVCodonGap(c, bNodes, i, j, k);
        }
    }

    // ************************************************************************
    /**
     * Window size
     */
    @Override
    protected int getWindowSize()
    {
        return Codon.CODON_LEN + 1;
    }

    // ************************************************************************
    /**
     * SPECIFIC ALIGNERS FOR SUBMATRICES
     */
    @Override
    public Alignment getSubmatrixAligner(BackTranslationGraph sA, BackTranslationGraph sB, Alignment prev)
    {
        GlobalDPFSCGapAlignment al =
            new GlobalDPFSCGapAlignment(sA, sB, this.d, this.e, this.match, this.mismatch, this.codonSkip,
                this.codonSkipExtension, this.allowedFrameshifts);
        // al.allowedFrameshifts = this.allowedFrameshifts;
        al.setScoreMatrix(this.getScoreMatrix());
        if (prev != null) {
            // System.out.println(this.allowedFrameshifts + " "
            // + ((GlobalDPFSCGapAlignment) prev).getPerformedFrameshifts() + " " + al.allowedFrameshifts);
            al.allowedFrameshifts -= ((GlobalDPFSCGapAlignment) prev).getPerformedFrameshifts();
            // System.out.println("prev " + ((GlobalDPFSCGapAlignment)prev).getPerformedFrameshifts());
        }
        // System.out.println("FFFFFFFFFFFFF " + al.allowedFrameshifts);
        return al;
    }

    @Override
    public String traceBack(CellUnitKey k)
    {
        // TODO Auto-generated method stub
        return null;
    }
}
