package sequoia.path.alignment.spaceecon;

import sequoia.path.alignment.Alignment;
import sequoia.path.alignment.CellUnitKey;
import sequoia.path.alignment.Coord;
import sequoia.path.alignment.GlobalDPFSCGapAlignment;
import sequoia.path.graph.BackTranslationGraph;
import sequoia.path.graph.Node;
import sequoia.path.util.Codon;

public class SpaceEconomicGlobalDPFSCAligner extends SpaceEconomicFSCAligner
{
    // Seqence A labels the columns (so it is horizontal); it is traversed by index i (or c...)
    // Seqence B labels the rowq (so it is vertical ); it is traversed by index j
    // ************************************************************************
    // C O N S T R U C T O R S
    // public SpaceEconomicGlobalDPFSCAligner(BackTranslationGraph a, BackTranslationGraph b, int scoreAtLeast) {
    // super(a, b, scoreAtLeast);
    // }

    public SpaceEconomicGlobalDPFSCAligner(BackTranslationGraph A, BackTranslationGraph B, int d, int e, int m, int x,
        int codonSkip, int codonSkipExtend, int fsNo, int acceptedScore)
    {
        super(A, B, d, e, m, x, codonSkip, codonSkipExtend, fsNo, acceptedScore);
    }

    // ************************************************************************
    // I M P L E M E N T A T I O N S O F A B S T R A C T F U N C T I O N S
    @Override
    public double getCellUnitInitValue()
    {
        return Double.MIN_VALUE;
    }

    @Override
    public double getBorderInitValue(int where)
    {
        return (where / Codon.CODON_LEN) * this.codonSkip + (where % Codon.CODON_LEN) * this.d;
    }

    @Override
    public void initCellUnitStartField(ScanWindowCellUnitInfo c, int i, int j, CellUnitKey k)
    {
        c.setMyStart((new Coord(0, 0, Node.ANY_NODE, Node.ANY_NODE)));
    }

    @Override
    public void acknowledgeScore(int i, int j, CellUnitKey k)
    {
        if (this.win.windowToMatrixIdx(i) == this.A.size() && j == this.B.size()) {
            this.list.update(i, j, k, this);
        }
    }

    // ************************************************************************
    // SPECIFIC ALIGNERS FOR SUBMATRICES
    @Override
    public Alignment getFullmatrixAligner()
    {
        GlobalDPFSCGapAlignment al =
            new GlobalDPFSCGapAlignment(this.A, this.B, this.d, this.e, this.match, this.mismatch, this.codonSkip,
                this.codonSkipExtension, this.allowedFrameshifts);
        al.setScoreMatrix(this.getScoreMatrix());
        al.allowedFrameshifts = this.allowedFrameshifts;
        return al;
    }
}
