package sequoia.path.graph;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Helper class used during graph transformations (e.g. subgraph extraction, reverse complementary), mapping nodes from
 * the original graph to their corresponding nodes in the transformed graph, in order to be able to recreate the links.
 */
class GraphTransformationCorrespondence
{
    ArrayList<HashMap<Node, Node>> nodeCorrespondenceMap;

    GraphTransformationCorrespondence(int size)
    {
        this.nodeCorrespondenceMap = new ArrayList<HashMap<Node, Node>>(size);
        for (int i = 0; i < size; ++i) {
            this.nodeCorrespondenceMap.add(new HashMap<Node, Node>());
        }
    }

    void put(int position, Node key, Node value)
    {
        if (key.symbol != Node.ANY) {
            this.nodeCorrespondenceMap.get(position).put(key, value);
        }
    }

    Node get(int position, Node key)
    {
        if (position < 0) {
            return Node.ANY_NODE;
        }
        if (position >= this.nodeCorrespondenceMap.size()) {
            return null;
        }
        return (key.symbol == Node.ANY) ? key : this.nodeCorrespondenceMap.get(position).get(key);
    }

    @Override
    public String toString()
    {
        StringBuilder str = new StringBuilder();
        for (int i = 0; i < this.nodeCorrespondenceMap.size(); ++i) {
            str.append("\t" + i + ": ");
            str.append(this.nodeCorrespondenceMap.get(i));
            str.append("\n");
        }
        return str.toString();
    }
}
