package sequoia.path.main;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import sequoia.path.alignment.Alignment;
import sequoia.path.alignment.estimation.ScoreEvaluationParametersAlignment;
import sequoia.path.alignment.filter.AlignmentFilter;
import sequoia.path.alignment.format.AlignmentFormatter;
import sequoia.path.alignment.format.PlainTextFormatter;
import sequoia.path.alignment.format.XMLFormatter;
import sequoia.path.params.ParamProcessor;
import sequoia.path.params.Parameters;

public class Main
{
	public static Parameters params = new Parameters();

    public static void main(String args[]) throws Exception
    {
        // Thread t = new Thread(new Stopper());
        // t.start();

        if (args.length == 0) {
            // load a config file
            try {
                BufferedReader br = new BufferedReader(new FileReader("config"));
                args = br.readLine().split(" ");
            } catch (Exception ex) {
                System.err.println(ex.getClass() + " Parameters not provided.");
                return;
            }
        }
        execute(args);
        // t.stop();

    }

    public static void execute(String args[]) throws Exception
    {
        params = new ParamProcessor().parse(args);

        if (params.estimator != null) {
            estimate();
        } else {
        	align();
        }
    }
    
    protected static void estimate(){
    	 params.estimator.setStatsOutputFile(params.databaseFile);

         Alignment a = null;
         while (params.getData().hasMoreAlignments()) {
             try {
            	 a = params.getData().nextAlignment();
            	 System.out.println("---");
	             a.align();
	             params.estimator.extractLocalAlignments((ScoreEvaluationParametersAlignment) a);
             } catch (Exception ex) {
                 ex.printStackTrace();
             }
         }
         params.estimator.compute();
         params.closeOutput();
    }
    
    protected static Alignment align() {
    	Alignment a = null;
    	AlignmentFormatter formatter;
    	PrintStream xmlOut = params.getXmlOut();
    	int p = 0, t = params.getData().alignmentsCount();
    	if (xmlOut != null) {
    		formatter = XMLFormatter.instance;
    	} else {
    		formatter = PlainTextFormatter.instance;
    	}
    	if (params.getXmlOut() != null) {
    		xmlOut.println("<?xml version=\"1.0\"?>");
    		xmlOut.println("<pathresult>");
    	}
        if (params.showProgressBar()) {
        	ProgressBar.displayProgress(p, t);
        }
        while (params.getData().hasMoreAlignments()) {
            try {
                a = params.getData().nextAlignment();
                a.align();
                if (xmlOut != null) {
                	xmlOut.print(formatter.format(a));
                } else {
                	boolean accepted = true;
                	for(AlignmentFilter filter : params.filters) {
                		accepted = accepted && filter.accepts(a);
                	}
	                if (accepted && params.getSelOut() != null) {
	            		params.getSelOut().println(a.traceBack() + "\n");
	                }
                }
                if (params.isVerbose()) {
                	System.out.println(a.traceBack());
                    System.out.println("---");
                }
                if (params.showProgressBar()) {
                	ProgressBar.displayProgress(++p, t);
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        if (params.getXmlOut() != null) {
    		xmlOut.println("</pathresult>");
    	}
        params.closeOutput();
        
        return a;
    }

    
    protected static class Stopper implements Runnable
    {
        public static final int TIMEOUT_HOUR = 11;

        public static final int TIMEOUT_MINUTE = 20;

        @SuppressWarnings("deprecation")
        public void run()
        {
            Date now = new Date();
            Date timeout = new Date();
            timeout.setHours(TIMEOUT_HOUR);
            timeout.setMinutes(TIMEOUT_MINUTE);
            if (now.after(timeout)) {
                timeout.setDate(timeout.getDate() + 1);
            }
            long sleep = timeout.getTime() - now.getTime();
            try {
                System.out.println("Sleeping for " + (sleep / 1000) + " seconds");
                Thread.sleep(sleep);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            System.exit(0);
        }
    }
    
    protected static class ProgressBar {
        static int p1 = -1, p2;
        static final int MAX = 60;
        static long startTime = System.currentTimeMillis();
    	
        static final char COMPLETED_SYMBOL = '=', UNCOMPLETED_SYMBOL = ' ', LAST_COMPLETED_SYMBOL = '>';
      	public static void reset() {
    	    p1 = -1;
    	    startTime = System.currentTimeMillis();
    	}
    	
        public static void displayProgress(int p, int total) {
        	//if (p == 0 || (p = (p + 1)*100/total)  > p1) {
        	int _p = (p)*100/total;
        		p1 = _p * MAX / 100;
        		System.out.print("\r\033[36;1m[");
        		for (p2 = 0; p2 < p1; ++p2) {
        			System.out.print(COMPLETED_SYMBOL);
        		}
        		if (p1 > 0 && p1 != MAX) {
        			System.out.print("\b" + LAST_COMPLETED_SYMBOL);
        		}
        		for (p2 = p1; p2 < MAX; ++p2) {
        			System.out.print(UNCOMPLETED_SYMBOL);
        		}
        		System.out.print("]");
        		System.out.printf(" %3d%%   (%d/%d)",  Math.min(_p, 100), p, total);
        		long eta = (System.currentTimeMillis() - startTime) / 1000;
        		int s = (int) (eta % 60);
        		int m = (int) (eta / 60) % 60;
        		int h = (int) (eta/3600);
        		System.out.printf("    ETA: "+ (h>0? (h + "h") : "") + ((h>0 || m>0) ? String.format("%02dm", m) : "") + String.format("%02ds", s)  +"\033[0m");
        		System.out.flush();
        		if (p1 == MAX) {
        			System.out.println();
        		}
        	//	p1 = p;
        	//}
        }
    }
}
