package sequoia.path.scorematrixbuilder.directcomputation;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;

import sequoia.path.scorematrixbuilder.matrix.SquareMatrix;
import sequoia.path.scorematrixbuilder.mutation.CodonMutationRate;
import sequoia.path.scores.MatrixAnalyzer;
import sequoia.path.util.GeneticCodeMap;

public class Compute
{
    public static void main(String args[])
    {
        int separation = ContextScoreMatrixBuilder.DEFAULT_MATRIX_SEPARATION;
        ArrayList<Double> ages = new ArrayList<Double>();
        ArrayList<String> labels = new ArrayList<String>();

        for (String arg : args) {
            if (arg.startsWith("-s")) {
                try {
                    separation = Integer.parseInt(arg.substring(2));
                } catch (Exception ex) {
                }
            } else {
                try {
                    ages.add(Double.parseDouble(arg));
                } catch (Exception e) {
                    System.out.println("Not a numerical value: " + arg);
                    labels.add(arg);
                }
            }
        }

        for (double age : ages) {
            try {
                System.out.println(age + " ............... ");
                System.out.println((String.format("Creating contextMatrix%05.2f.file ...", age)));
                createMatrix(age, separation);
            } catch (Exception ex) {
                System.err.println("Failed to create matrix for age " + age);
                ex.printStackTrace();
            }
        }

        for (String label : labels) {
            try {
                System.out.println(label + " ............... ");
                PrintStream metaOut;
                try {
                    metaOut = new PrintStream(String.format("mat_%s.out", label));
                } catch (FileNotFoundException e) {
                    metaOut = System.out;
                }
                createMatrix(String.format("cmpm%s.mat", label), String.format("contextMatrix%s.file", label), metaOut,
                    separation, "TRANSLATION DEPENDENT SCORES (" + label + ")");
            } catch (Exception ex) {
                System.err.println("Failed to create matrix for age " + label);
                ex.printStackTrace();
            }
        }
    }

    private static void createMatrix(double age, int separation)
    {
        PrintStream metaOut;
        try {
            metaOut = new PrintStream(String.format("mat_%05.2f.out", age));
        } catch (FileNotFoundException e) {
            metaOut = System.out;
        }
        createMatrix(String.format("cmpm%05.2f.mat", age), String.format("contextMatrix%05.2f.file", age), metaOut,
            separation, "TRANSLATION DEPENDENT SCORES, evolutionary distance " + age + "(mutations / codon)");
    }

    private static void createMatrix(String mutationProbsFileName, String outputFilename, PrintStream metaOut,
        int separation, String comment)
    {

        SquareMatrix codonMutationProb =
            SquareMatrix.loadFromSimpleFile(ClassLoader.getSystemResourceAsStream(mutationProbsFileName));

        ContextScoreMatrixBuilder builder =
            new ContextScoreMatrixBuilder(GeneticCodeMap.getInstance().getCodons(), codonMutationProb,
                CodonMutationRate.pi, separation);
        MatrixAnalyzer z = new MatrixAnalyzer();
        z.analyseMatrix(builder.getScoreMatrix(), metaOut, comment);
        metaOut.flush();

        try {
            ObjectOutputStream oos =
                new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(outputFilename)));
            oos.writeObject(builder.getScoreMatrix());
            oos.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
