package sequoia.path.scorematrixbuilder.directcomputationcondensed;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;

import sequoia.path.scorematrixbuilder.matrix.SquareMatrix;
import sequoia.path.scorematrixbuilder.mutation.CodonMutationRate;
import sequoia.path.scores.MatrixAnalyzer;
import sequoia.path.util.Codon;
import sequoia.path.util.GeneticCodeMap;

public class Compute
{
    public static void main(String args[])
    {
        int separation = ContextScoreMatrixBuilder.DEFAULT_MATRIX_SEPARATION;
        ArrayList<Double> ages = new ArrayList<Double>();

        for (String arg : args) {
            if (arg.startsWith("-s")) {
                try {
                    separation = Integer.parseInt(arg.substring(2));
                } catch (Exception ex) {
                }
            } else {
                try {
                    ages.add(Double.parseDouble(arg));
                } catch (Exception e) {
                    System.out.println("Wrong argument: " + arg);
                }
            }
        }

        for (double age : ages) {
            try {
                System.out.println(age + " ............... ");
                // System.out.println((String.format("%s%05.2f%s", "cmpm", age, ".mat")));
                System.out.println((String.format("Creating contextMatrix%05.2f.file ...", age)));
                createMatrix(age, separation);
            } catch (Exception ex) {
                System.err.println("Failed to create matrix for age " + age);
                ex.printStackTrace();
            }
        }
    }

    private static void createMatrix(double age, int separation)
    {
        PrintStream metaOut;
        try {
            metaOut = new PrintStream(String.format("meta%05.2f.out", age));
        } catch (FileNotFoundException e) {
            metaOut = System.out;
        }
        createMatrix(String.format("cmpm%05.2f.mat", age), String.format("contextMatrix%05.2f.file", age), metaOut,
            separation);
    }

    private static void createMatrix(String mutationProbsFileName, String outputFilename, PrintStream metaOut,
        int separation)
    {

        SquareMatrix codonMutationProb =
            SquareMatrix.loadFromSimpleFile(ClassLoader.getSystemResourceAsStream(mutationProbsFileName));
        // System.out.println(displayMutationProbs(codonMutationProb));
        // System.out.println(displaySums(codonMutationProb));
        // System.exit(0);

        ContextScoreMatrixBuilder builder =
            new ContextScoreMatrixBuilder(GeneticCodeMap.getInstance().getCodons(), codonMutationProb,
                CodonMutationRate.pi, separation);
        new MatrixAnalyzer().analyseMatrix(builder.getScoreMatrix(), System.out, ""); // TODO

        try {
            ObjectOutputStream oos =
                new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(outputFilename)));
            oos.writeObject(builder.getScoreMatrix());
            oos.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static String displayMutationProbs(SquareMatrix m)
    {
        StringBuilder str = new StringBuilder();

        Codon codons[] = GeneticCodeMap.getInstance().getCodons();

        str.append("       ");
        for (Codon c : codons) {
            str.append(String.format("%7s", c));
        }
        str.append("\n");

        for (int i = 0; i < m.size(); ++i) {
            str.append(codons[i] + " ");
            for (int j = 0; j < m.size(); ++j) {
                str.append(String.format(" %6.4f", m.get(i, j)));
            }
            str.append("\n");
        }

        return str.toString();
    }

    private static String displaySums(SquareMatrix m)
    {
        StringBuilder strR = new StringBuilder("ROWS (i)\n");
        StringBuilder strC = new StringBuilder("COLUMNS (j)\n");

        double sum = 0;

        Codon codons[] = GeneticCodeMap.getInstance().getCodons();
        double sumrows[] = new double[m.size()];
        Arrays.fill(sumrows, 0);
        double sumcols[] = new double[m.size()];
        Arrays.fill(sumcols, 0);
        for (int i = 0; i < m.size(); ++i) {
            for (int j = 0; j < m.size(); ++j) {
                sumrows[i] += m.get(i, j);
                sumcols[j] += m.get(i, j);
            }

        }

        for (int k = 0; k < m.size(); ++k) {
            strR.append(codons[k] + ": ");
            strC.append(codons[k] + ": ");
            strR.append(sumrows[k]);
            strC.append(sumcols[k]);
            strR.append("\n");
            strC.append("\n");
            sum += sumrows[k];
        }
        return strR.toString() + "\n" + strC.toString() + "\nsum = " + sum;
    }
}
