package sequoia.path.scorematrixbuilder.directcomputationcondensed;

import java.util.Arrays;

import sequoia.path.scorematrixbuilder.mutation.CodonMutationRate;
import sequoia.path.util.Codon;
import sequoia.path.util.GeneticCodeMap;
import sequoia.path.util.GenomicSequenceUtils;

public class Test
{
    Codon[] allCodons = GeneticCodeMap.getInstance().getCodons();

    double codonFreqs[] = CodonMutationRate.pi;

    double baseFreqs[] = new double[GenomicSequenceUtils.ALPHABET_SIZE];

    public static void main(String[] args)
    {
        Test t = new Test();
        // Codon c = new Codon("TTT", 1, 'W');
        // int overlapping[] = getOverlappingCodons(c, 0, 2, 1, false);
        // for (int co : overlapping) {
        // p += getOverlapProbability(c.getID(), co, overlapping) * getMutationProbability(c,
        // c1)*getMutationProbability(co, c2.getID());
        // }

        // t.listOverlappingCodons("ACC", 1, 0, false);
        // t.listOverlappingCodons("TCC", 1, 0, true);
        // int overlapZoneSize = Codon.CODON_LEN - (int)Math.round((Math.log(16) /
        // Math.log(GenomicSequenceUtils.ALPHABET_SIZE)));
        // System.out.println(overlapZoneSize);
        // Codon c1 = new Codon("CAT", 0.5, 'H');
        // Codon c2 = new Codon("GAT", 0.5, 'D');
        //		
        // Codon c1 = new Codon("AGT", 1.0, 'M');
        // Codon c2 = new Codon("GAT", 0.5, 'D');
        //		

        // t.checkEqual(t.showOverlaps(c1, c2, 1, 0, 2, false), t.showOverlaps(c2,
        // c1, 0, 1, 2, false));

        // t.checkSum1(4);
        // t.checkSum1(5);

        t.checkSum1B(1);
        t.checkSum1B(2);
        t.checkSum1B(3);
    }

    Test()
    {
        computeBaseFreqs();
    }

    void computeBaseFreqs()
    {
        Arrays.fill(this.baseFreqs, 0);
        double sum = 0;
        for (int i = 0; i < this.allCodons.length; ++i) {
            for (int j = 0; j < Codon.CODON_LEN; ++j) {
                this.baseFreqs[GenomicSequenceUtils.getNucleotideID(this.allCodons[i].getSymbol(j))] +=
                    this.codonFreqs[i];
                this.baseFreqs[GenomicSequenceUtils.getNucleotideID(GenomicSequenceUtils
                    .getComplementary(this.allCodons[i].getSymbol(j)))] += this.codonFreqs[i];
                sum += 2 * this.codonFreqs[i];
            }
        }
        for (int i = 0; i < this.baseFreqs.length; ++i) {
            this.baseFreqs[i] /= sum;
            // System.err.print(" " + baseFreqs[i]);
        }
    }

    private void checkSum1B(int commonLen)
    {
        double sum = 0;
        int start = Codon.CODON_LEN - commonLen;
        for (Codon c1 : this.allCodons) {
            int overlaps[] = new ContextScoreMatrixBuilder().getOverlappingCodons(c1, start, 0, commonLen, false);
            for (int ci : overlaps) {
                double p = this.codonFreqs[c1.getID()] * this.codonFreqs[ci];
                ;
                for (int i = start; i < Codon.CODON_LEN; ++i) {
                    p /= this.baseFreqs[GenomicSequenceUtils.getNucleotideID(c1.getSymbol(i))];
                }
                sum += p;
            }
        }
        System.out.println(sum);
    }

    private void checkSum1(int size)
    {
        double sum = 0;
        int count = (int) Math.pow(GenomicSequenceUtils.ALPHABET_SIZE, size);
        for (int id = 0; id < count; ++id) {
            double p = 1;
            int ID = id;
            // decompose
            for (int i = 0; i < size; ++i) {
                p *= this.baseFreqs[ID % GenomicSequenceUtils.ALPHABET_SIZE];
                ID /= GenomicSequenceUtils.ALPHABET_SIZE;
            }
            sum += p;
        }

        System.out.println(sum);

    }

    /**
     * @param codon
     * @param from1
     * @param from2
     * @param rc
     */
    private void listOverlappingCodons(String codon, int from1, int from2, boolean rc)
    {
        // if(rc) {
        // if (from1 > 0) {
        // from1 = Codon.CODON_LEN - from1 - 1;
        // } else {
        // from2 = Codon.CODON_LEN - from2 - 1;
        // }
        // }
        int len = Math.min(Codon.CODON_LEN - from1, Codon.CODON_LEN - from2);
        int codons[] =
            new ContextScoreMatrixBuilder().getOverlappingCodons(new Codon(codon, 1, 'X'), from1, from2, len, rc);

        System.out.println("Overlappings for " + codon + " from (" + from1 + ", " + from2 + ")" + " on " + len
            + " positions" + (rc ? ", reverse complementary" : "") + ":");
        for (int i : codons) {
            System.out.println(String.format("%2d: %s", i, this.allCodons[i]));
        }
    }

    private CodonPair[] showOverlaps(Codon c1, Codon c2, int from1, int from2, int len, boolean rc)
    {
        CodonPair pairs[] =
            new CodonPair[this.allCodons.length
                * (int) Math.pow(GenomicSequenceUtils.ALPHABET_SIZE, Codon.CODON_LEN - len)];
        int i = 0;
        for (Codon c : this.allCodons) {
            int overlapping[] = new ContextScoreMatrixBuilder().getOverlappingCodons(c, from1, from2, len, rc);
            for (int co : overlapping) {
                pairs[i++] = new CodonPair(c, this.allCodons[co]);
            }
        }
        Arrays.sort(pairs);
        for (CodonPair p : pairs) {
            System.out.println(p);
        }
        System.out.println();
        System.out.println();
        return pairs;
    }

    private void checkEqual(CodonPair p1[], CodonPair p2[])
    {
        if (p1.length != p2.length) {
            System.out.println("Different lengths! " + p1.length + " " + p2.length);
        }

        for (int i = 0; i < p1.length; ++i) {
            CodonPair cp1 = p1[i];
            CodonPair cp2 = p2[i];
        }
        // for (int i = 0, j = 0; i < p1.length && j < p2.length;) {
        // CodonPair c1 = p1[i];
        // CodonPair c2 = p2[j];
        // if (c1.compareTo(c2) == 0) {
        // ++i;
        // ++j;
        // } else if (c1.compareTo(c2) < 0) {
        // System.out.println("Different! " + c1 + " != " + c2);
        // ++i;
        // } else {
        // System.out.println("Different! " + c1 + " != " + c2);
        // ++j;
        // }
        // }
    }

    class CodonPair implements Comparable<CodonPair>
    {
        final Codon c1, c2;

        CodonPair(Codon C1, Codon C2)
        {
            // if (C1.compareTo(C2) < 0) {
            this.c1 = C1;
            this.c2 = C2;
            // } else {
            // c1 = C2;
            // c2 = C1;
            // }
        }

        @Override
        public int hashCode()
        {
            final int PRIME = 31;
            int result = 1;
            result = PRIME * result + ((this.c1 == null) ? 0 : this.c1.hashCode());
            result = PRIME * result + ((this.c2 == null) ? 0 : this.c2.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj)
        {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final CodonPair other = (CodonPair) obj;
            if (this.c1 == null) {
                if (other.c1 != null) {
                    return false;
                }
            } else if (!this.c1.equals(other.c1)) {
                return false;
            }
            if (this.c2 == null) {
                if (other.c2 != null) {
                    return false;
                }
            } else if (!this.c2.equals(other.c2)) {
                return false;
            }
            return true;
        }

        public int compareTo(CodonPair other)
        {
            if (this.c1.compareTo(other.c1) < 0) {
                return -1;
            }
            if (this.c1.compareTo(other.c1) > 0) {
                return 1;
            }
            return this.c2.compareTo(other.c2);
        }

        @Override
        public String toString()
        {
            return "(" + this.c1.toString() + ", " + this.c2.toString() + ")";
        }
    }

}
