package sequoia.path.scorematrixbuilder.matrix;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Arrays;

public class SquareMatrix
{

    protected double matrix[][];

    public int size()
    {
        return this.matrix.length;
    }

    void addAt(int i, int j, double value)
    {
        this.matrix[i][j] += value;
    }

    void setAt(int i, int j, double value)
    {
        this.matrix[i][j] = value;
    }

    void mulAt(int i, int j, double value)
    {
        this.matrix[i][j] *= value;
    }

    public double get(int i, int j)
    {
        return this.matrix[i][j];
    }

    public SquareMatrix(int size)
    {
        this.matrix = new double[size][size];
    }

    public SquareMatrix(SquareMatrix data)
    {
        this(data.size());
        for (int i = 0; i < size(); ++i) {
            for (int j = 0; j < size(); ++j) {
                this.matrix[i][j] = data.get(i, j);
            }
        }

    }

    public SquareMatrix(int size, boolean I)
    {
        this.matrix = new double[size][size];
        if (I) {
            for (int i = 0; i < size; ++i) {
                this.matrix[i][i] = 1;
            }
        }
    }

    public SquareMatrix mul(SquareMatrix m)
    {
        if (this.size() != m.size()) {
            return null;
        }

        SquareMatrix p = new SquareMatrix(this.size());

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                for (int k = 0; k < this.size(); ++k) {
                    p.addAt(i, j, this.matrix[i][k] * m.get(k, j));
                }
            }
        }
        return p;
    }

    public SquareMatrix selfMul(SquareMatrix m)
    {
        if (this.size() != m.size()) {
            return null;
        }

        SquareMatrix c = this.clone();
        this.clear();

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                for (int k = 0; k < this.size(); ++k) {
                    this.matrix[i][j] += c.get(i, k) * m.get(k, j);
                }
            }
        }
        return this;
    }

    public SquareMatrix mul(double val)
    {
        SquareMatrix p = new SquareMatrix(this.size());

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                p.setAt(i, j, this.matrix[i][j] * val);
            }
        }
        return p;
    }

    public SquareMatrix selfMul(double val)
    {
        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                this.matrix[i][j] *= val;
            }
        }
        return this;
    }

    public SquareMatrix add(SquareMatrix m)
    {
        if (this.size() != m.size()) {
            return null;
        }

        SquareMatrix s = new SquareMatrix(this.size());

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                s.setAt(i, j, this.matrix[i][j] + m.get(i, j));
            }
        }
        return s;
    }

    public SquareMatrix selfAdd(SquareMatrix m)
    {
        if (this.size() != m.size()) {
            return null;
        }

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                this.matrix[i][j] += m.get(i, j);
            }
        }
        return this;
    }

    public double maxDif(SquareMatrix m)
    {
        if (this.size() != m.size()) {
            return Double.MAX_VALUE;
        }
        double max = 0, dif;

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                dif = Math.abs(this.matrix[i][j] - m.get(i, j));
                if (dif > max) {
                    max = dif;
                }
            }
        }
        return max;
    }

    public boolean acceptableDif(SquareMatrix m, double max)
    {
        if (this.size() != m.size()) {
            return false;
        }

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                if (Math.abs(this.matrix[i][j] - m.get(i, j)) > max) {
                    return false;
                }
            }
        }
        return true;
    }

    public void clear()
    {
        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                this.matrix[i][j] = 0;
            }
        }
    }

    public void copyFrom(SquareMatrix m)
    {
        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                this.matrix[i][j] = m.get(i, j);
            }
        }
    }

    public void translate(double val)
    {
        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                this.matrix[i][j] += val;
            }
        }
    }

    @Override
    public SquareMatrix clone()
    {
        SquareMatrix c = new SquareMatrix(this.size());

        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                c.setAt(i, j, this.matrix[i][j]);
            }
        }
        return c;
    }

    @Override
    public String toString()
    {
        StringBuilder str = new StringBuilder();
        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                str.append(this.matrix[i][j]);
                str.append('\t');
            }
            str.append('\n');
        }
        return str.toString();
    }

    public String exportOctaveFormat()
    {
        StringBuilder str = new StringBuilder();
        str.append("[ ");
        for (int i = 0; i < this.size(); ++i) {
            for (int j = 0; j < this.size(); ++j) {
                str.append(this.matrix[i][j]);
                str.append(", ");
            }
            str.append("; ");
        }
        str.append("]");
        return str.toString();
    }

    @Override
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + Arrays.hashCode(this.matrix);
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SquareMatrix other = (SquareMatrix) obj;
        if (!Arrays.equals(this.matrix, other.matrix)) {
            return false;
        }
        return true;
    }

    public static SquareMatrix getI(int size)
    {
        SquareMatrix m = new SquareMatrix(size);
        for (int i = 0; i < size; ++i) {
            m.matrix[i][i] = 1;
        }
        return m;
    }

    public static SquareMatrix loadFromSimpleFile(InputStream is)
    {
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(is));
            String line = br.readLine();
            int i = 0, j = 0;
            SquareMatrix m = null;
            while (line != null) {
                line = line.trim();
                String tks[] = line.split("\\s+");
                if (m == null) {
                    m = new SquareMatrix(tks.length);
                    // System.err.println("Matrix size: " + m.size());
                }
                for (j = 0; j < m.matrix[i].length; ++j) {
                    m.matrix[i][j] = Double.parseDouble(tks[j]);
                }
                line = br.readLine();
                ++i;
            }
            br.close();
            // System.err.println(m);
            return m;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }

}
