package sequoia.path.scorematrixbuilder.mutation;

import sequoia.path.util.Codon;
import sequoia.path.util.GeneticCodeMap;

public class CodonMutationRate
{

    // order: TTT, ... paml
    // public static double pi[] = {0.0098, 0.0175, 0.0022, 0.0144, 0.0084, 0.0150, 0.0019, 0.0124, 0.0106, 0.0189,
    // 0.0000, 0.0000, 0.0047, 0.0083, 0.0000, 0.0069, 0.0159, 0.0283, 0.0036, 0.0234, 0.0137, 0.0243, 0.0031, 0.0201,
    // 0.0172, 0.0306, 0.0039, 0.0253, 0.0076, 0.0135, 0.0017, 0.0112, 0.0157, 0.0279, 0.0036, 0.0231, 0.0135, 0.0240,
    // 0.0031, 0.0198, 0.0170, 0.0302, 0.0038, 0.0250, 0.0075, 0.0133, 0.0017, 0.0110, 0.0253, 0.0448, 0.0057, 0.0370,
    // 0.0217, 0.0385, 0.0049, 0.0318, 0.0273, 0.0485, 0.0062, 0.0401, 0.0121, 0.0214, 0.0027, 0.0177};
    // order: AAA ...
    public static double pi[] =
        {0.02823, 0.02061, 0.03188, 0.01824, 0.01687, 0.01596, 0.00589, 0.01439, 0.01350, 0.01906, 0.01185, 0.01362,
        0.00936, 0.01996, 0.02187, 0.01747, 0.01367, 0.01437, 0.03368, 0.01159, 0.01776, 0.01499, 0.00588, 0.01760,
        0.00618, 0.00871, 0.00849, 0.00542, 0.00803, 0.01734, 0.03733, 0.01502, 0.03109, 0.02459, 0.03887, 0.02517,
        0.01790, 0.02126, 0.00596, 0.01995, 0.01844, 0.01761, 0.01327, 0.01155, 0.00837, 0.01376, 0.02651, 0.01359,
        0.00027, 0.01599, 0.00014, 0.01320, 0.01334, 0.01597, 0.00432, 0.01712, 0.00032, 0.01190, 0.01246, 0.01098,
        0.00852, 0.01957, 0.01413, 0.01907};

    protected double k = 5.0;

    protected double omega = 0.3;

    double Q[][];

    CodonMutationRate()
    {
        Codon codons[] = GeneticCodeMap.getInstance().getCodons();
        this.Q = new double[codons.length][codons.length];
        for (int i = 0; i < this.Q.length; ++i) {
            Codon ci = codons[i];
            for (int j = 0; j < i; ++j) {
                Codon cj = codons[j];
                if (ci.getAA() == Codon.STOP_CODON || cj.getAA() == Codon.STOP_CODON
                    || CodonRelation.getNucleotideDifsCount(ci, cj) > 1) {
                    this.Q[i][j] = 0;
                    this.Q[j][i] = 0;
                    continue;
                }
                this.Q[i][j] = pi[j];
                this.Q[j][i] = pi[i];
                if (CodonRelation.getCodonDif(ci, cj) == CodonRelation.CODON_DIF.NONSYN) {
                    this.Q[i][j] *= this.omega;
                    this.Q[j][i] *= this.omega;
                }
                if (CodonRelation.getHighestNucleotideDif(ci, cj) == CodonRelation.NUCLEOTIDE_DIF.TRANS) {
                    this.Q[i][j] *= this.k;
                    this.Q[j][i] *= this.k;
                }

            }
        }
        for (int i = 0; i < this.Q.length; ++i) {
            this.Q[i][i] = 0;
            for (int j = 0; j < this.Q[i].length; ++j) {
                if (j != i) {
                    this.Q[i][i] -= this.Q[i][j];
                }
            }
        }
        double normalizationFactor = 0;
        for (int i = 0; i < this.Q.length; ++i) {
            for (int j = 0; j < this.Q[i].length; ++j) {
                if (j != i) {
                    normalizationFactor += pi[i] * this.Q[i][j];
                }
            }
        }
        for (int i = 0; i < this.Q.length; ++i) {
            for (int j = 0; j < this.Q[i].length; ++j) {
                this.Q[i][j] /= normalizationFactor;
            }
        }
    }

    CodonMutationRate(boolean dummy)
    {
        Codon codons[] = GeneticCodeMap.getInstance().getCodons();
        this.Q = new double[codons.length][codons.length];
        for (int i = 0; i < this.Q.length; ++i) {
            for (int j = 0; j < i; ++j) {
                this.Q[i][j] = 0.1;
                this.Q[j][i] = 0.1;
            }
        }
        for (int i = 0; i < this.Q.length; ++i) {
            this.Q[i][i] = 0;
            for (int j = 0; j < this.Q[i].length; ++j) {
                if (j != i) {
                    this.Q[i][i] -= this.Q[i][j];
                }
            }
        }
    }

    @Override
    public String toString()
    {
        StringBuilder str = new StringBuilder();
        for (double[] row : this.Q) {
            for (double q : row) {
                str.append(String.format("%21.18f ", q));
            }
            str.append("\n");
        }
        return str.toString();
    }

    public String toOctaveString()
    {
        StringBuilder str = new StringBuilder();
        for (double[] row : this.Q) {
            for (double q : row) {
                str.append(String.format("%21.18f, ", q));
            }
            str.append(";\n");
        }
        return "Q = [\n" + str.toString() + "];\n";
    }

    public String toDummyString()
    {
        StringBuilder str = new StringBuilder();
        for (double[] row : this.Q) {
            for (double q : row) {
                str.append(String.format("%4.2f, ", q));
            }
            str.append(";\n");
        }
        return "[\n" + str.toString() + "];\n";
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {
        System.out.println(new CodonMutationRate().toOctaveString());
    }

}
