package sequoia.path.scorematrixbuilder.mutation;

import sequoia.path.util.Codon;
import sequoia.path.util.GeneticCodeMap;
import sequoia.path.util.GenomicSequenceUtils;

public class CodonRelation
{
    private static Codon[] codons = GeneticCodeMap.getInstance().getCodons();

    public static enum CODON_DIF
    {
        IDENTICAL, SYN, NONSYN
    };

    public static enum NUCLEOTIDE_DIF
    {
        IDENTICAL, TRANS, TRANSV
    };

    public static NUCLEOTIDE_DIF[] getNucleotideDif(int i1, int i2)
    {
        return getNucleotideDif(codons[i1], codons[i2]);
    }

    public static NUCLEOTIDE_DIF[] getNucleotideDif(Codon c1, Codon c2)
    {
        NUCLEOTIDE_DIF[] difs = new NUCLEOTIDE_DIF[Codon.CODON_LEN];
        for (int i = 0; i < Codon.CODON_LEN; ++i) {
            difs[i] = getNucleotideDif(c1.getSymbol(i), c2.getSymbol(i));
        }
        return difs;
    }

    public static CODON_DIF getCodonDif(int i1, int i2)
    {
        return getCodonDif(codons[i1], codons[i2]);
    }

    public static CODON_DIF getCodonDif(Codon c1, Codon c2)
    {
        if (c1.getSeq().equals(c2.getSeq())) {
            return CODON_DIF.IDENTICAL;
        } else if (c1.getAA() == c2.getAA()) {
            return CODON_DIF.SYN;
        } else {
            return CODON_DIF.NONSYN;
        }
    }

    public static NUCLEOTIDE_DIF getNucleotideDif(char n1, char n2)
    {
        if (n1 == n2) {
            return NUCLEOTIDE_DIF.IDENTICAL;
        } else if (GenomicSequenceUtils.isTransitionMutation(n1, n2)) {
            return NUCLEOTIDE_DIF.TRANS;
        } else {
            return NUCLEOTIDE_DIF.TRANSV;
        }
    }

    public static int getNucleotideDifsCount(Codon c1, Codon c2)
    {
        NUCLEOTIDE_DIF[] difs = new NUCLEOTIDE_DIF[Codon.CODON_LEN];
        for (int i = 0; i < Codon.CODON_LEN; ++i) {
            difs[i] = getNucleotideDif(c1.getSymbol(i), c2.getSymbol(i));
        }
        int difcount = 0;
        for (NUCLEOTIDE_DIF d : difs) {
            if (d != NUCLEOTIDE_DIF.IDENTICAL) {
                ++difcount;
            }
        }
        return difcount;
    }

    public static NUCLEOTIDE_DIF getHighestNucleotideDif(Codon c1, Codon c2)
    {
        NUCLEOTIDE_DIF dif = getNucleotideDif(c1.getSymbol(0), c2.getSymbol(0));
        for (int i = 1; i < Codon.CODON_LEN; ++i) {
            NUCLEOTIDE_DIF idif = getNucleotideDif(c1.getSymbol(i), c2.getSymbol(i));
            if (idif == NUCLEOTIDE_DIF.TRANSV || dif == NUCLEOTIDE_DIF.IDENTICAL) {
                dif = idif;
            }
        }
        return dif;
    }
}
