package sequoia.path.scores;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Formatter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Map.Entry;

import sequoia.path.util.Codon;
import sequoia.path.util.GeneticCodeMap;

public class CPSScoreMatrix
{
    class ScoreCell
    {
        public final char AA;

        public final char N;

        public final int pos;

        private double freq = 0;

        ScoreCell(char aa, char n, int i)
        {
            AA = aa;
            N = n;
            pos = i;
        }

        @Override
        public int hashCode()
        {
            final int prime = 31;
            int result = 1;
            result = prime * result + AA;
            result = prime * result + N;
            result = prime * result + pos;
            return result;
        }

        @Override
        public boolean equals(Object obj)
        {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final ScoreCell other = (ScoreCell) obj;
            if (AA != other.AA) {
                return false;
            }
            if (N != other.N) {
                return false;
            }
            if (pos != other.pos) {
                return false;
            }
            return true;
        }

        @Override
        public String toString()
        {
            StringBuilder str = new StringBuilder();
            str.append(AA + " ");
            str.append(pos + " ");
            str.append(N + " ");
            str.append(freq + " ");
            return str.toString();
        }

        public double getFreq()
        {
            return freq;
        }

        void setFreq(double freq)
        {
            this.freq = freq;
        }
    }

    public static final String Ns = "acgt_";

    public static final int a = 0;

    public static final int c = 1;

    public static final int g = 2;

    public static final int t = 3;

    public static final int _ = 4;

    public static final int n = _ + 1;

    // ARNDCQEGHILKMFPSTWYVZBJX
    public static final String AAs = "ARNDCQEGHILKMFPSTWYVZBJX";

    public static final int A = 0;

    public static final int R = 1;

    public static final int N = 2;

    public static final int D = 3;

    public static final int C = 4;

    public static final int Q = 5;

    public static final int E = 6;

    public static final int G = 7;

    public static final int H = 8;

    public static final int I = 9;

    public static final int L = 10;

    public static final int K = 11;

    public static final int M = 12;

    public static final int F = 13;

    public static final int P = 14;

    public static final int S = 15;

    public static final int T = 16;

    public static final int W = 17;

    public static final int Y = 18;

    public static final int V = 19;

    public static final int Z = 10;

    public static final int B = 21;

    public static final int J = 22;

    public static final int X = 23;

    public static final int AA = 20;// X + 1;

    double score[][][][][][] = new double[AA][Codon.CODON_LEN][n][AA][Codon.CODON_LEN][n];

    double aaBestScore[][] = new double[AA][AA];

    double aaMediumScore[][] = new double[AA][AA];

    public CPSScoreMatrix()
    {
        for (int ia1 = 0; ia1 < AA; ++ia1) {
            for (int ip1 = 0; ip1 < Codon.CODON_LEN; ++ip1) {
                for (int in1 = 0; in1 < n; ++in1) {
                    for (int ia2 = 0; ia2 < AA; ++ia2) {
                        for (int ip2 = 0; ip2 < Codon.CODON_LEN; ++ip2) {
                            for (int in2 = 0; in2 < n; ++in2) {
                                score[ia1][ip1][in1][ia2][ip2][in2] = Double.NEGATIVE_INFINITY;
                            }
                        }
                    }
                }
            }
        }
        Set<Entry<String,ArrayList<Codon>>> AAs = GeneticCodeMap.getInstance().A2CentrySet();
        Iterator<Entry<String, ArrayList<Codon>>> it = AAs.iterator();
        ArrayList<ScoreCell> cells = new ArrayList<ScoreCell>();
        while (it.hasNext()) {
            String AA = it.next().getKey();
            if (!GeneticCodeMap.getInstance().isNotAASet(AA)) {
                continue;
            }
            cells.addAll(computeFreqs(GeneticCodeMap.getInstance().getCodons(AA)));
        }
        try {
            PrintWriter p = new PrintWriter(new OutputStreamWriter(new FileOutputStream("cells.txt")));
            for (int i = 0; i < cells.size(); ++i) {
                p.println(cells.get(i).toString() + " ");
            }
            p.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

        for (int k = 0; k < cells.size(); ++k) {
            for (int l = k; l < cells.size(); ++l) {
                computeScore(cells.get(k), cells.get(l));
            }
        }

        computeAAMatrix();
    }

    private void computeAAMatrix()
    {
        for (int ia1 = 0; ia1 < AA; ++ia1) {
            if (!GeneticCodeMap.getInstance().isNotAASet(AAs.charAt(ia1) + "")) {
                continue;
            }
            for (int ia2 = ia1; ia2 < AA; ++ia2) {
                if (!GeneticCodeMap.getInstance().isNotAASet(AAs.charAt(ia2) + "")) {
                    continue;
                }
                computeAAScore(AAs.charAt(ia1), AAs.charAt(ia2));
            }
        }
    }

    private void computeAAScore(char a, char b)
    {
        ArrayList<Codon> aCodons = GeneticCodeMap.getInstance().getCodons(a);
        ArrayList<Codon> bCodons = GeneticCodeMap.getInstance().getCodons(b);
        double maxScore = Double.NEGATIVE_INFINITY;
        double medScore = 0;
        for (Iterator<Codon> ita = aCodons.iterator(); ita.hasNext();) {
            Codon ac = ita.next();
            for (Iterator<Codon> itb = bCodons.iterator(); itb.hasNext();) {
                Codon bc = itb.next();
                double crtScore = getUngappedAlignmentScore(ac, bc);
                if (crtScore > maxScore) {
                    maxScore = crtScore;
                }
                medScore += crtScore * ac.getFrequency() * bc.getFrequency();
            }
        }
        aaBestScore[getAIndex(a)][getAIndex(b)] = maxScore;
        aaBestScore[getAIndex(b)][getAIndex(a)] = maxScore;
        aaMediumScore[getAIndex(a)][getAIndex(b)] = medScore;
        aaMediumScore[getAIndex(b)][getAIndex(a)] = medScore;
    }

    private int getAIndex(char a)
    {
        try {
            return this.getClass().getField((a + "").toUpperCase()).getInt(this);
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
        return X;
    }

    private int getNIndex(char a)
    {
        try {
            return this.getClass().getField((a + "").toLowerCase()).getInt(this);
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
        return _;
    }

    private double getUngappedAlignmentScore(Codon a, Codon b)
    {
        double s = 0;
        for (int i = 0; i < Codon.CODON_LEN; ++i) {
            s +=
                score[getAIndex(a.getAA())][i][getNIndex(a.getSymbol(i))][getAIndex(b.getAA())][i][getNIndex(b
                    .getSymbol(i))];
        }
        return s;
    }

    public void printScoreMatrix(PrintWriter out)
    {
        out.print("         ");

        for (int ia2 = 0; ia2 < AA; ++ia2) {
            for (int ip2 = 0; ip2 < Codon.CODON_LEN; ++ip2) {
                for (int in2 = 0; in2 < n; ++in2) {
                    out.print(new Formatter().format("    %c %d %c  ", AAs.charAt(ia2), ip2, Ns.charAt(in2)));
                }
            }
        }
        out.println();
        for (int ia1 = 0; ia1 < AA; ++ia1) {
            for (int ip1 = 0; ip1 < Codon.CODON_LEN; ++ip1) {
                for (int in1 = 0; in1 < n; ++in1) {
                    out.print(new Formatter().format("%c %d %c   ", AAs.charAt(ia1), ip1, Ns.charAt(in1)));
                    for (int ia2 = 0; ia2 < AA; ++ia2) {
                        for (int ip2 = 0; ip2 < Codon.CODON_LEN; ++ip2) {
                            for (int in2 = 0; in2 < n; ++in2) {
                                out.print(new Formatter().format("%10.3g ", score[ia1][ip1][in1][ia2][ip2][in2]));
                            }
                        }
                    }
                    out.println();
                }
            }
        }
        out.flush();
    }

    public void printAABestScoreMatrix(PrintWriter out)
    {
        out.print("     ");

        for (int ia2 = 0; ia2 < AA; ++ia2) {
            out.print(new Formatter().format("    %c  ", AAs.charAt(ia2)));
        }
        out.println();
        for (int ia1 = 0; ia1 < AA; ++ia1) {
            out.print(new Formatter().format("%c   ", AAs.charAt(ia1)));
            for (int ia2 = 0; ia2 < AA; ++ia2) {
                out.print(new Formatter().format("%6.3g ", aaBestScore[ia1][ia2]));
            }
            out.println();
        }
        out.flush();
    }

    public void printAAMediumScoreMatrix(PrintWriter out)
    {
        out.print("     ");

        for (int ia2 = 0; ia2 < AA; ++ia2) {
            out.print(new Formatter().format("    %c  ", AAs.charAt(ia2)));
        }
        out.println();
        for (int ia1 = 0; ia1 < AA; ++ia1) {
            out.print(new Formatter().format("%c   ", AAs.charAt(ia1)));
            for (int ia2 = 0; ia2 < AA; ++ia2) {
                out.print(new Formatter().format("%6.3g ", aaMediumScore[ia1][ia2]));
            }
            out.println();
        }
        out.flush();
    }

    public void printAAMedDiffMatrix(PrintWriter out, SimpleScoreMatrix ref)
    {
        printAADiffMatrix(out, aaMediumScore, ref);
    }

    public void printAABestDiffMatrix(PrintWriter out, SimpleScoreMatrix ref)
    {
        printAADiffMatrix(out, aaBestScore, ref);
    }

    private void printAADiffMatrix(PrintWriter out, double mat[][], SimpleScoreMatrix ref)
    {
        // PreMatrix bm = new PreMatrix("blos.txt");
        out.print("     ");

        for (int ia2 = 0; ia2 < AA; ++ia2) {
            out.print(new Formatter().format("    %c  ", AAs.charAt(ia2)));
        }
        out.println();
        for (int ia1 = 0; ia1 < AA; ++ia1) {
            out.print(new Formatter().format("%c   ", AAs.charAt(ia1)));
            for (int ia2 = 0; ia2 < AA; ++ia2) {
                out.print(new Formatter().format("%6.3g ", mat[ia1][ia2] - ref.score[ia1][ia2]));
            }
            out.println();
        }
        out.flush();
    }

    public void printAAMedSignMatrix(PrintWriter out, SimpleScoreMatrix ref)
    {
        printAASignMatrix(out, aaMediumScore, ref);
    }

    public void printAABestSignMatrix(PrintWriter out, SimpleScoreMatrix ref)
    {
        printAASignMatrix(out, aaBestScore, ref);
    }

    private void printAASignMatrix(PrintWriter out, double mat[][], SimpleScoreMatrix ref)
    {
        // PreMatrix bm = new PreMatrix("blos.txt");
        out.print("     ");
        int count = 0;
        int sign;

        for (int ia2 = 0; ia2 < AA; ++ia2) {
            out.print(new Formatter().format("%c  ", AAs.charAt(ia2)));
        }
        out.println();
        for (int ia1 = 0; ia1 < AA; ++ia1) {
            out.print(new Formatter().format("%c   ", AAs.charAt(ia1)));
            for (int ia2 = 0; ia2 < AA; ++ia2) {
                sign = (int) (Math.signum(mat[ia1][ia2]) * Math.signum(ref.score[ia1][ia2] + 0.01));
                out.print(new Formatter().format("%2d ", sign));
                if (sign == -1) {
                    ++count;
                }
            }
            out.println();
        }
        out.println(count);
        out.flush();
    }

    private void computeScore(ScoreCell a, ScoreCell b)
    {
        double s = Math.log(a.freq * b.freq);
        if (a.N == b.N) {
            s += 4;
        } else if (transMutation(a.N, b.N)) {
            s -= 1.5;
        } else {
            s -= 4;
        }
        int in1 = 0, in2 = 0, ia1 = 0, ia2 = 0, ip1 = a.pos, ip2 = b.pos;
        if (a.AA == Codon.START_CODON || a.AA == Codon.STOP_CODON || b.AA == Codon.START_CODON
            || b.AA == Codon.STOP_CODON)
        {
            return;
        }
        in1 = getNIndex(a.N);
        in2 = getNIndex(b.N);
        ia1 = getAIndex(a.AA);
        ia2 = getAIndex(b.AA);
        score[ia1][ip1][in1][ia2][ip2][in2] = s;
        score[ia2][ip2][in2][ia1][ip1][in1] = s;
    }

    private boolean transMutation(char a, char b)
    {
        if (a > b) {
            return transMutation(b, a);
        }
        return a == 'A' && b == 'G' || a == 'C' && b == 'T';
    }

    private ArrayList<ScoreCell> computeFreqs(ArrayList<Codon> codons)
    {
        HashMap<ScoreCell, Double> crtCells = new HashMap<ScoreCell, Double>();
        for (int i = 0; i < Codon.CODON_LEN; ++i) {
            for (int j = 0; j < codons.size(); ++j) {
                Codon codon = codons.get(j);
                ScoreCell u = new ScoreCell(codon.getAA(), codon.getSymbol(i), i);
                Double f = crtCells.get(u);
                double val = 0;
                if (f != null) {
                    val = f.doubleValue();
                }
                val += codon.getFrequency();
                crtCells.put(u, new Double(val));
            }
        }
        ArrayList<ScoreCell> cells = new ArrayList<ScoreCell>(crtCells.size());
        Iterator<ScoreCell> it = crtCells.keySet().iterator();
        while (it.hasNext()) {
            ScoreCell c = it.next();
            c.setFreq(crtCells.get(c).doubleValue());
            cells.add(c);
        }
        return cells;
    }
}
