package sequoia.path.scores;

import java.io.Serializable;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Set;

import sequoia.path.util.Codon;
import sequoia.path.util.GeneticCodeMap;
import sequoia.path.util.GenomicSequenceUtils;

public class CompressedContextScoreMatrix extends HashMap<SymbolPairAlignmentData, AnnotatedScore> implements
    Serializable, ScoreMatrix
{

    public static final long serialVersionUID = 20080721;

    public CompressedContextScoreMatrix(ContextScoreMatrix sm)
    {
//        System.out.println();
//        System.out.println();
//        System.out.println();
//        System.out.println("BUILDING COMPRESSED SCORE MATRIX........");
//        System.out.println();
//        System.out.println();
//        System.out.println();
//        System.out.println();
//        System.out.println();
//        System.out.println();
//        System.out.println();
//        System.out.println();
        Codon[] codons = GeneticCodeMap.getInstanceAmbiguous().getCodons();

        for (Codon c1 : codons) {
            // System.out.println("..." + c1 + "...");
            char aa1 = c1.getAA();
            for (int p1 = 0; p1 < Codon.CODON_LEN; ++p1) {
                char n1 = c1.getSymbol(p1);
                SymbolData sd1 = new SymbolData(n1, aa1, p1, false);
                SymbolData sd1rc = sd1.reverseComplementary();
                for (Codon c2 : codons) {
                    // System.out.println("   ..." + c2 + "...");
                    char aa2 = c2.getAA();
                    for (int p2 = 0; p2 < Codon.CODON_LEN; ++p2) {
                        char n2 = c2.getSymbol(p2);
                        SymbolData sd2 = new SymbolData(n2, aa2, p2, false);
                        SymbolData sd2rc = sd2.reverseComplementary();
                        put(new SymbolPairAlignmentData(sd1, sd2), new AnnotatedScore(Double.NEGATIVE_INFINITY, sd1
                            .getN(), sd2.getN()));
                        put(new SymbolPairAlignmentData(sd2, sd1), new AnnotatedScore(Double.NEGATIVE_INFINITY, sd2
                            .getN(), sd1.getN()));
                        // put (new SymbolPairAlignmentData(sd1rc, sd2), new
                        // AnnotatedScore(Double.NEGATIVE_INFINITY,
                        // sd1rc.getN(), sd2.getN()));
                        // put (new SymbolPairAlignmentData(sd2rc, sd1), new
                        // AnnotatedScore(Double.NEGATIVE_INFINITY,
                        // sd2rc.getN(), sd1.getN()));
                        put(new SymbolPairAlignmentData(sd1, sd2rc), new AnnotatedScore(Double.NEGATIVE_INFINITY, sd1
                            .getN(), sd2rc.getN()));
                        put(new SymbolPairAlignmentData(sd2, sd1rc), new AnnotatedScore(Double.NEGATIVE_INFINITY, sd2
                            .getN(), sd1rc.getN()));
                    }
                }
            }
            // System.out.println("..." + c1 + "...");
        }

        //System.out.println();

        for (SymbolPairAlignmentData spd : this.keySet()) {
            double score = getScore(spd);
            SymbolData s1 = spd.getS1();
            SymbolData s2 = spd.getS2();
            Set<Character> set1 = GenomicSequenceUtils.getExtendedSymbolSet(s1.getN());
            Set<Character> set2 = GenomicSequenceUtils.getExtendedSymbolSet(s2.getN());
            char aa1 = s1.getAA();
            char aa2 = s2.getAA();
            int p1 = s1.getPos();
            int p2 = s2.getPos();
            boolean rc1 = s1.isRC();
            boolean rc2 = s2.isRC();
            for (char n1 : set1) {
                for (char n2 : set2) {
                    SymbolData ds1 = new SymbolData(n1, aa1, p1, rc1);
                    SymbolData ds2 = new SymbolData(n2, aa2, p2, rc2);
                    try {
                        double dscore = sm.get(new SymbolPairAlignmentData(ds1, ds2));
                        if (dscore > score) {
                            put(spd, dscore, n1, n2);
                            score = dscore;
                        }
                    } catch (NullPointerException ex) {
                    	if (aa1 != '>' && aa2 != '>' && aa1 != 'U' && aa2 != 'U') {
	                        System.err.println("    >>> " + ds1 + " " + ds2);
	                        System.err.println("    >>> " + s1 + " " + s2);
                    	}
                        // System.exit(0);
                    }
                }
            }
            //System.out.println("..." + spd + "...");
        }
//        System.out.println();
//        System.out.println();
//        System.out.println();
    }

    public AnnotatedScore get(SymbolPairAlignmentData spd)
    {
        return super.get(spd);
    }

    public AnnotatedScore get(SymbolData s1, SymbolData s2)
    {
        return get(new SymbolPairAlignmentData(s1, s2));
    }

    public double getScore(SymbolPairAlignmentData spd)
    {
        double val = Double.NEGATIVE_INFINITY;
        AnnotatedScore as = get(spd);
        if (as != null) {
            val = as.getScore();
        }
        return val;
    }

    public double getScore(SymbolData s1, SymbolData s2)
    {
        return getScore(new SymbolPairAlignmentData(s1, s2));
    }

    public String getNPair(SymbolPairAlignmentData spd)
    {
        AnnotatedScore as = get(spd);
        if (as != null) {
            return as.getN1() + "" + as.getN2();
        }
        return spd.getN1() + "" + spd.getN2();
    }

    public String getNPair(SymbolData s1, SymbolData s2)
    {
        return getNPair(new SymbolPairAlignmentData(s1, s2));
    }

    public String getNPair(char n1, char aa1, int p1, boolean rc1, char n2, char aa2, int p2, boolean rc2)
    {
        return getNPair(new SymbolData(n1, aa1, p1, rc1), new SymbolData(n2, aa2, p2, rc2));
    }

    public double getScore(char n1, char aa1, int p1, boolean rc1, char n2, char aa2, int p2, boolean rc2)
    {
        return getScore(new SymbolData(n1, aa1, p1, rc1), new SymbolData(n2, aa2, p2, rc2));
    }

    public AnnotatedScore get(char n1, char aa1, int p1, boolean rc1, char n2, char aa2, int p2, boolean rc2)
    {
        return get(new SymbolData(n1, aa1, p1, rc1), new SymbolData(n2, aa2, p2, rc2));
    }

    protected void put(SymbolPairAlignmentData where, double score, char n)
    {
        put(where, new AnnotatedScore(score, n));
    }

    protected void put(SymbolPairAlignmentData where, double score, char n1, char n2)
    {
        put(where, new AnnotatedScore(score, n1, n2));
    }

    protected void put(SymbolData where1, SymbolData where2, double score, char n)
    {
        put(new SymbolPairAlignmentData(where1, where2), new AnnotatedScore(score, n));
    }

    protected void put(SymbolData where1, SymbolData where2, double score, char n1, char n2)
    {
        put(new SymbolPairAlignmentData(where1, where2), new AnnotatedScore(score, n1, n2));
    }
    
    public Set<SymbolPairAlignmentData> getAllSymbolPairs() {
    	return keySet();
    }
    
    public void set(SymbolPairAlignmentData spd, double score) {
    	put(spd, score, '?');
    }

    @Override
    public String toString()
    {
        Object[] okeys = keySet().toArray();
        SymbolPairAlignmentData keys[] = new SymbolPairAlignmentData[okeys.length];
        int i = 0;
        for (Object k : okeys) {
            keys[i++] = (SymbolPairAlignmentData) k;
        }
        Arrays.sort(keys);

        StringBuilder str = new StringBuilder();
        for (SymbolPairAlignmentData k : keys) {
            str.append(k.toString());
            str.append(" = ");
            str.append(get(k));
            str.append('\n');
        }
        return str.toString();
    }
}
