package sequoia.path.scores;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;

import sequoia.path.scores.ScoreMatrix;
import sequoia.path.scores.SymbolData;
import sequoia.path.scores.SymbolPairAlignmentData;
import sequoia.path.util.Codon;
import sequoia.path.util.GeneticCodeMap;
import sequoia.path.util.GenomicSequenceUtils;

public class MatrixAnalyzer {
	
	static final boolean rcstates[] = {false, true};

	public void analyseMatrix(ScoreMatrix m, PrintStream out, String comment) {
		ScoreMatrix mrc = (ScoreMatrix)m.clone();
		mrc.clear();
		ScoreMatrix mnrc = (ScoreMatrix)mrc.clone();
		for (SymbolPairAlignmentData s : m.getAllSymbolPairs()) {
			if (s.getS1().isRC() || s.getS2().isRC()) {
				mrc.set(s, (Double)m.get(s));
			}
			else {
				mnrc.set(s, (Double)m.get(s));
			}
		}
		printMetadata(out, m, comment);
		out.println(m);
		out.flush();
	}

	protected ArrayList<HashMap<String, Double> > printMetadata(PrintStream metaOut, ScoreMatrix matrix, String comment) {
		ArrayList<HashMap<String, Double> > metaMatrices = new ArrayList<HashMap<String,Double>>();

		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " " + comment);;
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER);

//		// average
//		metaOut.println("-------------------------------------------------------------");
//		metaOut.println("Average");
//		metaOut.println();
//		metaOut.println("Nucleotide projection");
//		metaMatrices.add(selectionMatrix(metaOut, matrix, false, new SymbolDataStopCodonFilter(),new SymbolDataNSelector(), new SymbolDataAgregatorSum()));
//		metaOut.println();
//		metaOut.println("Nucleotide|position projection");
//		metaMatrices.add(selectionMatrix(metaOut, matrix, false, new SymbolDataStopCodonFilter(),new SymbolDataNPosSelector(), new SymbolDataAgregatorSum()));
////		metaMatrices.add(selectionMatrix(metaOut, matrix, false, new SymbolDataStopCodonFilter(),new SymbolDataAASelector(), new SymbolDataAgregatorSum()));
////		metaOut.println();
//		metaOut.println("Amino-acid projection");
//		for (boolean rc : rcstates) {
//			for (int pos1 = 0; pos1 < Codon.CODON_LEN; ++pos1) {
//				for (int pos2 = 0; pos2 < Codon.CODON_LEN; ++pos2) {
//					metaOut.println("\n" + pos1 + "<->" + pos2 + (rc ? " RC" : ""));
//					selectionMatrix(metaOut, matrix, false, new SymbolDataPositionFilter(pos1, pos2, rc), new SymbolDataAASelector(), new SymbolDataAgregatorReplace());
//					metaOut.println();
//				}
//			}
//		}
//
//		metaOut.println();
//		metaOut.println();


		// expectation
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " -------------------------------------------------------------");
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Nucleotide projection");
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER );
		metaMatrices.add(selectionMatrix(metaOut, matrix, true, new SymbolDataStopCodonFilter(), new SymbolDataNSelector(), new SymbolDataAgregatorSum()));
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER );
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Nucleotide|position projection");
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER );
		metaMatrices.add(selectionMatrix(metaOut, matrix, true, new SymbolDataStopCodonFilter(), new SymbolDataNPosSelector(), new SymbolDataAgregatorSum()));      
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER );
		metaMatrices.add(selectionMatrix(metaOut, matrix, true, new SymbolDataStopCodonFilter(),new SymbolDataAASelector(), new SymbolDataAgregatorSum()));
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER );
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Amino-acid projection");
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER );
		metaOut.println("\n" + ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Same reading frame");
		selectionMatrix(metaOut, matrix, true, new SymbolDataSameRFFilter(), new SymbolDataAASelector(), new SymbolDataAgregatorSum());
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER );
		for (boolean rc : rcstates) {
			for (int pos1 = 0; pos1 < Codon.CODON_LEN; ++pos1) {
				for (int pos2 = 0; pos2 < Codon.CODON_LEN; ++pos2) {
					metaOut.println("\n" + ScoreMatrixFormatter.COMMENT_LINE_MARKER + " " + 
							pos1 + ScoreMatrixFormatter.SUBMATRIX_POSITION_MARKER + pos2 + (rc ? " " + ScoreMatrixFormatter.REVERSE_COMPLEMENTARY_MARKER : ""));
					selectionMatrix(metaOut, matrix, true, new SymbolDataPositionFilter(pos1, pos2, rc), new SymbolDataAASelector(), new SymbolDataAgregatorReplace());
//					//metaOut.println("Nucleotide|position projection");
//					//metaMatrices.add(selectionMatrix(metaOut, matrix, true, new SymbolDataStopCodonFilter(), new SymbolDataNPosSelector(), new SymbolDataAgregatorSum()));      
//
				}
			}
		}
//		metaOut.println("Greatest difference between the scores of various nucleotide pairs in each AA pair on the given positions");
//		for (boolean rc : rcstates) {
//			for (int pos1 = 0; pos1 < Codon.CODON_LEN; ++pos1) {
//				for (int pos2 = 0; pos2 < Codon.CODON_LEN; ++pos2) {
//					metaOut.println("\n" + pos1 + "<->" + pos2 + (rc ? " RC" : ""));
//					selectionMatrix(metaOut, matrix, true, new SymbolDataPositionFilter(pos1, pos2, rc), new SymbolDataAASelector(), new SymbolDataAgregatorMaxDif());
//					//metaOut.println("Nucleotide|position projection");
//					//metaMatrices.add(selectionMatrix(metaOut, matrix, true, new SymbolDataStopCodonFilter(), new SymbolDataNPosSelector(), new SymbolDataAgregatorSum()));      
//
//				}
//			}
//		}
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER);
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER);

		printStats(metaOut, matrix);

		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER);
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER);
		
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " -------------------------------------------------------------");
		
		return metaMatrices;

	}
	
//	protected void printMatchTransTransvDiffs(PrintStream metaOut, ScoreMatrix matrix) {
//		metaOut.println("M / Ts / Tv  difs");
//		for (boolean rc : rcstates) {
//			for (int pos1 = 0; pos1 < Codon.CODON_LEN; ++pos1) {
//				for (int pos2 = 0; pos2 < Codon.CODON_LEN; ++pos2) {
//					metaOut.println("\n" + pos1 + "<->" + pos2 + (rc ? " RC" : ""));
//					SymbolDataFilter filter = new SymbolDataPositionFilter(pos1, pos2, rc);
//					SymbolDataFieldSelector fieldSelector = new SymbolDataAASelector();
//					
//					HashMap<String, Double> match = new HashMap<String, Double>();
//					HashMap<String, Double> transi = new HashMap<String, Double>();
//					HashMap<String, Double> transv = new HashMap<String, Double>();
//
//					for (SymbolPairAlignmentData k : matrix.getAllSymbolPairs()) {
//						if (!filter.filter(k)) {
//							continue;
//						}
//						String s = fieldSelector.selectFieldsLabel(k);
//						double score = matrix.getScore(k);
//						
//						
//					}
////					for (String k : sum.keySet()) {
////						if (count.get(k) == 0) {
////							System.err.println(k);
////						}
////						sum.put(k, sum.get(k) / count.get(k));
////					}
////
////					printMetaMartix(metaOut, sum, fieldSelector);
////
////					return sum;
////					
////					selectionMatrix(metaOut, matrix, true, new SymbolDataPositionFilter(pos1, pos2, rc), new SymbolDataAASelector(), new SymbolDataAgregatorMaxDif());
////					//metaOut.println("Nucleotide|position projection");
////					//metaMatrices.add(selectionMatrix(metaOut, matrix, true, new SymbolDataStopCodonFilter(), new SymbolDataNPosSelector(), new SymbolDataAgregatorSum()));      
//				}
//			}
//		}
//		metaOut.println();
//		metaOut.println();
//	}
	protected void printStats(PrintStream metaOut, ScoreMatrix matrix) {
		SymbolDataFilter filter = new SymbolDataStopCodonFilter();
		double min = Double.MAX_VALUE, minNS = Double.MAX_VALUE, max = Double.NEGATIVE_INFINITY, med = 0;
		int positives = 0;
		int negatives = 0;
		int zeros = 0;
		for (SymbolPairAlignmentData k : matrix.getAllSymbolPairs()) {
			double val = (Double)matrix.get(k);
			if (val > max) {
				max = val;
			}
			if (val < min) {
				min = val;
			}
			if (val < minNS && filter.filter(k)) {
				minNS = val;
			}
			if (val > 0) {
				positives++;
			} else if (val < 0) {
				negatives ++;
			} else {
				zeros++;
			}
			
			med += val;
		}
		med /= matrix.getAllSymbolPairs().size();
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Max:       " + max);
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Min:       " + minNS);
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Avg:       " + String.format("%.3f", med));
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Positives: " + positives);
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Negatives: " + negatives);
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Zeros:     " + zeros);
		metaOut.println(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " Entries: " + matrix.getAllSymbolPairs().size());

	}

	protected HashMap<String, Double> metaMatrixDif(HashMap<String, Double> a, HashMap<String, Double> b) {
		HashMap<String, Double> dif = new HashMap<String, Double>();
		for (String k : a.keySet()) {
			dif.put(k, a.get(k) - b.get(k));
		}
		return dif;
	}

	protected HashMap<String, Double> metaMatrixRatio(HashMap<String, Double> a, HashMap<String, Double> b) {
		HashMap<String, Double> dif = new HashMap<String, Double>();
		for (String k : a.keySet()) {
			dif.put(k, a.get(k) / b.get(k));
		}
		return dif;
	}

	private void printMetaMartix(PrintStream metaOut, HashMap<String, Double> m, SymbolDataFieldSelector fieldSelector) {
		HashSet<String> symbolsXSet = new HashSet<String>();
		HashSet<String> symbolsYSet = new HashSet<String>();

		for (String k : m.keySet()) {
			String labels[] = fieldSelector.parseFieldsLabel(k);
			symbolsXSet.add(labels[0]);
			symbolsYSet.add(labels[1]);

			//System.out.println(k + " " + labels[0] + " " + labels[1]);
		}

		Object[] objs = symbolsXSet.toArray();
		Arrays.sort(objs);
		String[] symbolsX = new String[objs.length];
		int i = 0;
		for (Object o : objs) {
			symbolsX[i++] = ((String)o);
		}
		Arrays.sort(symbolsX);

		objs = symbolsYSet.toArray();
		Arrays.sort(objs);
		String[] symbolsY = new String[objs.length];
		i = 0;
		for (Object o : objs) {
			symbolsY[i++] = ((String)o);
		}
		Arrays.sort(symbolsY);


		metaOut.print(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " ");
		metaOut.print(fieldSelector.emptyLabel());
		// purely for a more comprehensive display 
		String prevLabel = " ";
		for (String c : symbolsY) {
			metaOut.print(fieldSelector.colSeparator(prevLabel, c));
			metaOut.print(fieldSelector.displayLabel(c));
			prevLabel = c;
		}

		metaOut.println();
		metaOut.print(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " ");
		metaOut.println();
		//metaOut.print(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " ");
		// purely for a more comprehensive display 
		String prevLabelX = " ";
		for (String cx : symbolsX) {
			metaOut.print(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " ");
			String sep = fieldSelector.rowSeparator(prevLabelX, cx);
			metaOut.print(sep);
			if (sep.contains("\n")) {
				metaOut.print(ScoreMatrixFormatter.COMMENT_LINE_MARKER + " ");;
			}
			metaOut.print(fieldSelector.displayLabel(cx));
			prevLabelX = cx;
			String prevLabelY = " ";
			for (String cy : symbolsY) {
				metaOut.print(fieldSelector.colSeparator(prevLabelY, cy));
				metaOut.print(fieldSelector.displayNumber(m.get(cx + "" + cy)));
				prevLabelY = cy;
			}
			metaOut.println();
			//metaOut.print(ScoreMatrixFormatter.COMMENT_LINE_MARKER);

		}
	}

	public HashMap<String, Double> selectionMatrix(PrintStream metaOut, ScoreMatrix matrix, boolean weighted, 
			SymbolDataFilter filter, SymbolDataFieldSelector fieldSelector, SymbolDataAgregator dataAgregator) {
		HashMap<String, Double> sum = new HashMap<String, Double>();
		HashMap<String, Double> count = new HashMap<String, Double>();

		for (SymbolPairAlignmentData k : matrix.getAllSymbolPairs()) {
			if (!filter.filter(k)) {
				continue;
			}
			String s = fieldSelector.selectFieldsLabel(k);
			double score = matrix.getScore(k);
			double weight = weighted ? weight(k) : 1;
			dataAgregator.update(sum, count, s, score, weight);
		}


		for (String k : sum.keySet()) {
			if (count.get(k) == 0) {
				//System.err.println(k);
				sum.put(k, 0.0);
			} else {
				sum.put(k, sum.get(k) / count.get(k));
			}
		}

		printMetaMartix(metaOut, sum, fieldSelector);

		return sum;
	}
	
	/**
	 * 
	 * @param metaOut
	 * @param matrix
	 * @param label
	 */
	public void printMap(PrintStream metaOut, HashMap<SymbolPairAlignmentData, Double> matrix, String label) {
		//for (boolean rc : rcstates) {
		//	for (int pos1 = 0; pos1 < Codon.CODON_LEN; ++pos1) {
		//		for (int pos2 = 0; pos2 < Codon.CODON_LEN; ++pos2) {
					metaOut.println("\n" + label);
					selectionMatrix(metaOut, matrix, true, new SymbolDataStopCodonFilter(), new SymbolDataAANSelector(8), new SymbolDataAgregatorReplace());
					//metaOut.println();
		//		}
		//	}
		//}
	}
	
	/**
	 * 
	 * @param metaOut
	 * @param matrix
	 * @param weighted
	 * @param filter
	 * @param fieldSelector
	 * @param dataAgregator
	 * @return
	 */
	private HashMap<String, Double> selectionMatrix(PrintStream metaOut, HashMap<SymbolPairAlignmentData, Double> matrix, boolean weighted, 
			SymbolDataFilter filter, SymbolDataFieldSelector fieldSelector, SymbolDataAgregator dataAgregator) {
		HashMap<String, Double> sum = new HashMap<String, Double>();
		HashMap<String, Double> count = new HashMap<String, Double>();

		for (SymbolPairAlignmentData k : matrix.keySet()) {
			if (!filter.filter(k)) {
				continue;
			}
			String s = fieldSelector.selectFieldsLabel(k);
			double score = matrix.get(k);
			double weight = weighted ? weight(k) : 1;
			dataAgregator.update(sum, count, s, score, weight);
		}


		for (String k : sum.keySet()) {
			if (count.get(k) == 0) {
				System.err.println(k);
			}
			sum.put(k, sum.get(k) / count.get(k));
		}

		printMetaMartix(metaOut, sum, fieldSelector);

		return sum;
	}

//	---------------------------------------------------------------------------------
	protected static double weight(SymbolPairAlignmentData s) {
		return weight(s.getS1()) * weight(s.getS2());
	}

	protected static double weight(SymbolData s) {
		ArrayList<Codon> allAACodons = GeneticCodeMap.getInstance().getCodons(s.getAA());
		char n = s.isRC() ? GenomicSequenceUtils.getComplementary(s.getN()) : s.getN();
		int pos = s.isRC() ? Codon.CODON_LEN - s.getPos() - 1 : s.getPos();
		double weight = 0;
		for (Codon c: allAACodons) {
			if (c.getSymbol(pos) == n) {
				weight += c.getFrequency();
			}
		}
		return weight;
	}

	//---------------------------------------------------------------------------------
	/**
	 * 
	 * 
	 *
	 */ 
	static interface SymbolDataFieldSelector {
		String selectFieldsLabel(SymbolPairAlignmentData s);
		String[] parseFieldsLabel(String label);
		String emptyLabel();
		String displayLabel(String label);
		String displayNumber(double value);
		String rowSeparator(String prevLabel, String crtLabel);
		String colSeparator(String prevLabel, String crtLabel);
	}

	static abstract class AbstractSymbolDataSelector implements SymbolDataFieldSelector {
		protected int decimals;
		protected int positions;
		
		public static char LABEL_SEPARATOR = '|';
		
		AbstractSymbolDataSelector() {
			decimals = 3;
			positions = 7;
		}
		
		AbstractSymbolDataSelector(int decimals) {
			this.decimals = decimals;
			this.positions = decimals + 4;
		}
		AbstractSymbolDataSelector(int decimals, int positions) {
			this.decimals = decimals;
			this.positions = positions;
		}
		
		public String[] parseFieldsLabel(String label){
			String[] labels = new String[2];
			int mid = label.length() / 2;
			labels[0] = label.substring(0, mid);
			labels[1] = label.substring(mid, label.length());
			return labels;
		}
		
		public String displayNumber(double value) {
			return /*value == 0 ? emptyLabel():*/ String.format( "%" + positions + "." + decimals + "f", value);
		}

		public String displayLabel(String label) {
			return String.format("%"+positions+"s", label);
		}
		

		public String emptyLabel() {
			return String.format("%"+positions+"s", " ");
		}
		
		protected boolean sameGroup(String label1, String label2) {
			return label1.length() == 1 || label1.charAt(0) == label2.charAt(0);
		}

		public String rowSeparator(String prevLabel, String crtLabel) {
			if (sameGroup(crtLabel, prevLabel)) {
				 return "";
			}
			return "\n";
		}
		public String colSeparator(String prevLabel, String crtLabel){
			if (sameGroup(crtLabel, prevLabel)) {
				 return "";
			}
			return "  ";
		}
	}

	static class SymbolDataNSelector extends AbstractSymbolDataSelector {
		SymbolDataNSelector() {
			super();
		}
		
		SymbolDataNSelector(int decimals) {
			super(decimals);
		}
		SymbolDataNSelector(int decimals, int positions) {
			super(decimals, positions);
		}
		public String selectFieldsLabel(SymbolPairAlignmentData s) {
			return Character.toUpperCase(s.getN1()) + "" + Character.toUpperCase(s.getN2());
		}
	}
	static class SymbolDataAASelector extends AbstractSymbolDataSelector {
		SymbolDataAASelector() {
			super();
		}
		
		SymbolDataAASelector(int decimals) {
			super(decimals);
		}
		SymbolDataAASelector(int decimals, int positions) {
			super(decimals, positions);
		}
		public String selectFieldsLabel(SymbolPairAlignmentData s) {
			return s.getAA1() + "" + s.getAA2();
		}
	}
	static class SymbolDataAAMutationTypeSelector extends AbstractSymbolDataSelector {
		SymbolDataAAMutationTypeSelector() {
			super();
		}
		
		SymbolDataAAMutationTypeSelector(int decimals) {
			super(decimals);
		}
		SymbolDataAAMutationTypeSelector(int decimals, int positions) {
			super(decimals, positions);
		}
		public String selectFieldsLabel(SymbolPairAlignmentData s) {
			String mutationType = LABEL_SEPARATOR + (s.getN1() == s.getN2() ? "Ma" : (GenomicSequenceUtils.isTransitionMutation(s.getN1(), s.getN2()) ? "Tr" : "Tv"));
			return s.getAA1() + mutationType + s.getAA2() + mutationType;
		}
	}
	static class SymbolDataNPosSelector extends AbstractSymbolDataSelector {
		SymbolDataNPosSelector() {
			super();
		}
		
		SymbolDataNPosSelector(int decimals) {
			super(decimals);
		}
		SymbolDataNPosSelector(int decimals, int positions) {
			super(decimals, positions);
		}
		public String selectFieldsLabel(SymbolPairAlignmentData s) {
			return Character.toUpperCase(s.getN1()) + "" + s.getPos1()+ "" + Character.toUpperCase(s.getN2()) + "" + s.getPos2();
		}
	}
	
	static class SymbolDataAANSelector extends AbstractSymbolDataSelector {
		SymbolDataAANSelector() {
			decimals = 2;
			positions = 7;
		}
		
		SymbolDataAANSelector(int decimals) {
			super(decimals);
		}
		SymbolDataAANSelector(int decimals, int positions) {
			super(decimals, positions);
		}
		
		public String selectFieldsLabel(SymbolPairAlignmentData s) {
			return "" + s.getN1() + LABEL_SEPARATOR + s.getAA1() + s.getN2()+ LABEL_SEPARATOR + s.getAA2();
		}
	}

	//---------------------------------------------------------------------------------
	/**
	 * 
	 * 
	 *
	 */ 
	static interface SymbolDataAgregator {
		void update(HashMap<String, Double> data, HashMap<String, Double> counter, String key, double value, double weight);
	}
	
	static class SymbolDataAgregatorSum implements SymbolDataAgregator{
		public void update(HashMap<String, Double> data, HashMap<String, Double> counter, String key, double value, double weight) {
			Double t = data.get(key);
			if (t == null) {
				data.put(key,  value * weight);
				counter.put(key,  weight);
			} else {
				data.put(key, t.doubleValue() + value * weight);
				counter.put(key, counter.get(key) + weight);
			}
		}
	}
	
	static class SymbolDataAgregatorReplace implements SymbolDataAgregator{
		public void update(HashMap<String, Double> data, HashMap<String, Double> counter, String key, double value, double weight) {
			data.put(key,  value);
			counter.put(key,  1.0);
		}
	}
	static class SymbolDataAgregatorMaxDif implements SymbolDataAgregator{
		HashMap<String, Double> max = new HashMap<String, Double>();
		HashMap<String, Double> min = new HashMap<String, Double>();
		
		public void update(HashMap<String, Double> data, HashMap<String, Double> counter, String key, double value, double weight) {
			data.put(key,  updateWithCondition(max, key, value,  1) -updateWithCondition(min, key, value, -1));
			counter.put(key,  1.0);
		}
		
		/**
		 * This is for updating the min a,d max maps, each holding the minimum and maximum values
		 * ever encountered for each (partial) key.
		 * @param map The map to be updated (min or max, both members of this class).
		 * @param key The key to update
		 * @param value The new found value
		 * @param sign A parameter intorduced for unitary code for both min and max. It is multiplied with the difference 
		 * between the new value, and the value found in the map. If the result is greater than 0, the map is updated. 
		 * Consequently, the parameter's value must be 1 for updating max and -1 for updating min.
		 * @return The value pointed by key after the update
		 */
		private double updateWithCondition(HashMap<String, Double> map, String key, double value, int sign) {
			Double old = map.get(key);
			if (old == null || sign * (value - old) > 0) {
				map.put(key, value);
			}
			return map.get(key);
		}
	}
	
	static class SymbolDataAgregatorMax implements SymbolDataAgregator{
		HashMap<String, Double> max = new HashMap<String, Double>();
		HashMap<String, Double> min = new HashMap<String, Double>();
		
		public void update(HashMap<String, Double> data, HashMap<String, Double> counter, String key, double value, double weight) {
			data.put(key,  updateWithCondition(max, key, value,  1) -updateWithCondition(min, key, value, -1));
			counter.put(key,  1.0);
		}
		
		/**
		 * This is for updating the min a,d max maps, each holding the minimum and maximum values
		 * ever encountered for each (partial) key.
		 * @param map The map to be updated (min or max, both members of this class).
		 * @param key The key to update
		 * @param value The new found value
		 * @param sign A parameter intorduced for unitary code for both min and max. It is multiplied with the difference 
		 * between the new value, and the value found in the map. If the result is greater than 0, the map is updated. 
		 * Consequently, the parameter's value must be 1 for updating max and -1 for updating min.
		 * @return The value pointed by key after the update
		 */
		private double updateWithCondition(HashMap<String, Double> map, String key, double value, int sign) {
			Double old = map.get(key);
			if (old == null || sign * (value - old) > 0) {
				map.put(key, value);
			}
			return map.get(key);
		}
	}
//	---------------------------------------------------------------------------------
	/**
	 * 
	 * 
	 *
	 */ 
	static interface SymbolDataFilter {
		boolean filter(SymbolPairAlignmentData s);
	}
	
	static class SymbolDataPositionFilter extends SymbolDataStopCodonFilter{
		final int p1, p2;
		final boolean rc;
		SymbolDataPositionFilter (int p1, int p2, boolean rc) {
			this.p1 = p1;
			this.p2 = p2;
			this.rc = rc;
		}
		public boolean filter(SymbolPairAlignmentData s){
			return super.filter(s) && (s.getPos1() == p1) && (s.getPos2() == p2) && (s.getS2().isRC() == rc);
			/*(s.getAA1() != Codon.STOP_CODON) && (s.getAA2() != Codon.STOP_CODON) && (s.getAA1() != 'U') && (s.getAA2() != 'U') && */
		}
	}
	
	static class SymbolDataSameRFFilter extends SymbolDataStopCodonFilter{
		SymbolDataSameRFFilter () {
		}
		public boolean filter(SymbolPairAlignmentData s){
			return super.filter(s) && (s.getPos1() == s.getPos2()) && (s.getS2().isRC() == false);
		}
	}
	
	static class SymbolDataStopCodonFilter implements SymbolDataFilter{
		public boolean filter(SymbolPairAlignmentData s){
			return (s.getAA1() != Codon.STOP_CODON) && (s.getAA2() != Codon.STOP_CODON) && (s.getAA1() != 'U') && (s.getAA2() != 'U');
		}
	}
}
