package sequoia.path.scores;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;

import sequoia.path.scores.MatrixAnalyzer.SymbolDataAANSelector;
import sequoia.path.scores.MatrixAnalyzer.SymbolDataFieldSelector;
import sequoia.path.scores.MatrixAnalyzer.SymbolDataFilter;
import sequoia.path.scores.MatrixAnalyzer.SymbolDataPositionFilter;
import sequoia.path.util.Codon;

public class ScoreMatrixFormatter {
	
    protected static String SUBMATRIX_POSITION_MARKER = "<->";
    protected static String REVERSE_COMPLEMENTARY_MARKER = "RC";
    protected static String COMMENT_LINE_MARKER = "#";
	
	public static String format(ScoreMatrix m) {
    	StringBuilder str = new StringBuilder();
    	boolean rcstates[] = {false, true};
    	
    	SymbolDataFilter filterPosition; 
    	
    	SymbolDataFieldSelector fieldSelector = new SymbolDataAANSelector();
    	
    	
		for (boolean rc : rcstates) {
			for (int pos1 = 0; pos1 < Codon.CODON_LEN; ++pos1) {
				for (int pos2 = 0; pos2 < Codon.CODON_LEN; ++pos2) {
					filterPosition = new SymbolDataPositionFilter(pos1, pos2, rc);
					
					str.append("\n" + pos1 + SUBMATRIX_POSITION_MARKER + pos2 + (rc ? " " + REVERSE_COMPLEMENTARY_MARKER : "") + "\n");
					
					HashMap<String, Double> labelsToValues = new HashMap<String, Double>();
					
					for (SymbolPairAlignmentData k : m.getAllSymbolPairs()) {
						if (!filterPosition.filter(k)) {
							continue;
						}
						String s = fieldSelector.selectFieldsLabel(k);
						double score = m.getScore(k);
						labelsToValues.put(s, score);
					}

					
					HashSet<String> symbolsXSet = new HashSet<String>();
					HashSet<String> symbolsYSet = new HashSet<String>();
					
					for (String k : labelsToValues.keySet()) {
						String labels[] = fieldSelector.parseFieldsLabel(k);
						symbolsXSet.add(labels[0]);
						symbolsYSet.add(labels[1]);
					}

					Object[] objs = symbolsXSet.toArray();
					Arrays.sort(objs);
					String[] symbolsX = new String[objs.length];
					int i = 0;
					for (Object o : objs) {
						symbolsX[i++] = ((String)o);
					}
					Arrays.sort(symbolsX);

					objs = symbolsYSet.toArray();
					Arrays.sort(objs);
					String[] symbolsY = new String[objs.length];
					i = 0;
					for (Object o : objs) {
						symbolsY[i++] = ((String)o);
					}
					Arrays.sort(symbolsY);
					
					str.append(fieldSelector.emptyLabel());
					// purely for a more comprehensive display 
					String prevLabel = " ";
					for (String c : symbolsY) {
						str.append(fieldSelector.colSeparator(prevLabel, c));
						str.append(fieldSelector.displayLabel(c));
						prevLabel = c;
					}

					str.append('\n');
					// purely for a more comprehensive display 
					String prevLabelX = " ";
					for (String cx : symbolsX) {
						str.append(fieldSelector.rowSeparator(prevLabelX, cx));
						str.append(fieldSelector.displayLabel(cx));
						prevLabelX = cx;
						String prevLabelY = " ";
						for (String cy : symbolsY) {
							str.append(fieldSelector.colSeparator(prevLabelY, cy));
							str.append(fieldSelector.displayNumber(labelsToValues.get(cx + "" + cy)));
							prevLabelY = cy;
						}
						str.append('\n');

					}
					str.append("\n");
				}
			}
		}
		return str.toString();
	}
}
