package sequoia.path.scores;

import java.io.Serializable;

import sequoia.path.util.Codon;
import sequoia.path.util.GenomicSequenceUtils;

public class SymbolData implements Comparable<SymbolData>, Serializable
{
    public static final long serialVersionUID = 20080720;

    private char N;

    private char AA;

    private int pos;

    private boolean RC;

    public SymbolData(char n, char aa, int i, boolean rc)
    {
        N = n;
        AA = aa;
        pos = i;
        RC = rc;
    }

    public char getAA()
    {
        return AA;
    }

    public char getN()
    {
        return N;
    }

    public int getPos()
    {
        return pos;
    }

    public boolean isRC()
    {
        return RC;
    }

    @Override
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + AA;
        result = PRIME * result + N;
        result = PRIME * result + (RC ? 1231 : 1237);
        result = PRIME * result + pos;
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SymbolData other = (SymbolData) obj;
        if (AA != other.AA) {
            return false;
        }
        if (N != other.N) {
            return false;
        }
        if (RC != other.RC) {
            return false;
        }
        if (pos != other.pos) {
            return false;
        }
        return true;
    }

    @Override
    public SymbolData clone()
    {
        return new SymbolData(N, AA, pos, RC);
    }

    public SymbolData reverseComplementary()
    {
        return new SymbolData(GenomicSequenceUtils.getComplementary(N), AA, Codon.CODON_LEN - 1 - pos, !RC);
    }

    private int compareFields(int a, int b)
    {
        return (int) Math.signum(a - b);
    }

    private int compareFields(char a, char b)
    {
        return (int) Math.signum(a - b);
    }

    private int compareFields(boolean a, boolean b)
    {
        if (a == b) {
            return 0;
        } else if (a == false) {
            return -1;
        }
        return 1;
    }

    public int compareTo(SymbolData o)
    {
        int c;
        c = compareFields(RC, o.RC);
        if (c != 0) {
            return c;
        } else {
            c = compareFields(AA, o.AA);
            if (c != 0) {
                return c;
            } else {
                c = compareFields(N, o.N);
                if (c != 0) {
                    return c;
                } else {
                    return compareFields(pos, o.pos);
                }
            }
        }
    }

    @Override
    public String toString()
    {
        return (RC ? "RC " : "   ") + AA + "[" + pos + "]:" + N;
    }
}
