package sequoia.path.scores;

import java.io.Serializable;

public class SymbolPairAlignmentData implements Comparable<SymbolPairAlignmentData>, Serializable
{
    public static final long serialVersionUID = 20080720;

    SymbolData s1, s2;

    public SymbolPairAlignmentData(char n1, char n2, char aa1, char aa2, int i1, int i2, boolean rc)
    {
        s1 = new SymbolData(n1, aa1, i1, false);
        s2 = new SymbolData(n2, aa2, i2, rc);
    }

    public SymbolPairAlignmentData(SymbolData S1, SymbolData S2)
    {
        s1 = S1.clone();
        s2 = S2.clone();
    }

    public SymbolData getS1()
    {
        return s1;
    }

    public SymbolData getS2()
    {
        return s2;
    }

    public char getAA1()
    {
        return s1.getAA();
    }

    public char getAA2()
    {
        return s2.getAA();
    }

    public char getN1()
    {
        return s1.getN();
    }

    public char getN2()
    {
        return s2.getN();
    }

    public int getPos1()
    {
        return s1.getPos();
    }

    public int getPos2()
    {
        return s2.getPos();
    }

    @Override
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        // result = PRIME * result + (reverseCompl ? 1231 : 1237);
        result = PRIME * result + ((s1 == null) ? 0 : s1.hashCode());
        result = PRIME * result + ((s2 == null) ? 0 : s2.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SymbolPairAlignmentData other = (SymbolPairAlignmentData) obj;
        // if (reverseCompl != other.reverseCompl)
        // return false;
        if (s1 == null) {
            if (other.s1 != null) {
                return false;
            }
        } else if (!s1.equals(other.s1)) {
            return false;
        }
        if (s2 == null) {
            if (other.s2 != null) {
                return false;
            }
        } else if (!s2.equals(other.s2)) {
            return false;
        }
        return true;
    }

    public int compareTo(SymbolPairAlignmentData other)
    {
        int c;
        c = s1.compareTo(other.s1);
        if (c != 0) {
            return c;
        } else {
            c = s2.compareTo(other.s2);
            if (c != 0) {
                return c;
            }
            return 1;

        }
    }

    public boolean equals(SymbolPairAlignmentData obj, int seqIdx)
    {
        if (seqIdx == 1) {
            return s1.equals(obj.s1);
        } else if (seqIdx == 2) {
            return s2.equals(obj.s1);
        } else {
            return false;
        }
    }

    public SymbolPairAlignmentData getInverse()
    {
        return new SymbolPairAlignmentData(s2, s1);// , reverseCompl);
    }

    @Override
    public String toString()
    {
        return "(" + s1 + " | " + s2 + ')';
    }
}
