package sequoia.path.translation;

import sequoia.path.util.Codon;
import sequoia.path.util.DBSequence;
import sequoia.path.util.GeneticCodeMap;
import sequoia.path.util.GenomicSequenceUtils;

public class Translator
{
    public static String translate(String sequence, int readingFrame)
    {
        sequence = GenomicSequenceUtils.rna2dna(sequence);
        int length = getTranslatableSequenceEnd(sequence, readingFrame);
        StringBuilder result = new StringBuilder((sequence.length() - readingFrame) / Codon.CODON_LEN);
        for (int i = readingFrame; i < length; i += Codon.CODON_LEN) {
            result.append(GeneticCodeMap.getInstance().getAA(sequence.substring(i, i + Codon.CODON_LEN)));
        }
        return result.toString();
    }

    public static DBSequence translate(DBSequence sequence, int readingFrame)
    {
        return new DBSequence(translate(sequence.getSeq(), readingFrame), "P " + sequence.getId());
    }

    public static int getTranslatableSequenceEnd(String sequence, int readingFrame)
    {
        int result = sequence.length() - (sequence.length() - readingFrame) % Codon.CODON_LEN;
        if (result < 0) {
            result = Math.min(readingFrame, sequence.length());
        }
        if (result > sequence.length()) {
            result = sequence.length();
        }
        return result;
    }
}
