package sequoia.path.util;

public class Codon implements Comparable<Codon>
{
    public static final int CODON_LEN = 3;

    public static final char ANY_AA = 'X';

    public static final char START_CODON = '<';

    public static final char STOP_CODON = '>';

    String seq;

    double freq;

    char AA;

    Codon(String seq)
    {
        this(seq, 1, ANY_AA);
    }

    Codon(String seq, double freq)
    {
        this(seq, freq, ANY_AA);
    }

    Codon(String seq, char A)
    {
        this(seq, 1, A);
    }

    public Codon(String seq, double freq, char A)
    {
        this.seq = seq;
        this.freq = freq;
        this.AA = A;
    }

    public Codon createCopy()
    {
        return new Codon(this.seq, this.freq, this.AA);
    }

    public char getAA()
    {
        return this.AA;
    }

    public void setAA(char aa)
    {
        this.AA = aa;
    }

    public double getFrequency()
    {
        return this.freq;
    }

    public void setFrequency(double frequency)
    {
        this.freq = frequency;
    }

    public String getSeq()
    {
        return this.seq;
    }

    public char getSymbol(int pos)
    {
        return this.seq.charAt(pos);
    }

    public void setSeq(String seq)
    {
        this.seq = seq;
    }

    public int getID()
    {
        int id = 0;
        for (int i = 0; i < CODON_LEN; ++i) {
            id *= GenomicSequenceUtils.ALPHABET_SIZE;
            id += GenomicSequenceUtils.getNucleotideID(this.seq.charAt(i));
        }
        return id;
    }

    @Override
    public String toString()
    {
        return this.seq + "(" + this.AA + ")";
    }

    public int compareTo(Codon c)
    {
        return (int) Math.signum(this.getID() - c.getID());
    }

    @Override
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((this.seq == null) ? 0 : this.seq.toLowerCase().hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Codon other = (Codon) obj;
        if (this.seq == null) {
            if (other.seq != null) {
                return false;
            }
        } else if (!this.seq.equalsIgnoreCase(other.seq)) {
            return false;
        }
        return true;
    }
}
