package sequoia.path.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

public class GenomicSequenceUtils extends BaseSequenceUtils
{
    public static final HashMap<Character, HashSet<Character>> IUPACmapping =
        new HashMap<Character, HashSet<Character>>();

    public static final HashMap<HashSet<Character>, Character> IUPACreverseMapping =
        new HashMap<HashSet<Character>, Character>();

    public static final String BASIC_SYMBOLS = "ACGT";

    public static final int ALPHABET_SIZE = 4;

    public static final String EXTENDED_SYMBOLS = BASIC_SYMBOLS + "RYMKWSBDHVN";

    public static final char MATCH = '|';

    public static final char TRANSITION = ':';

    public static final char TRANSVERSION = '.';

    public static int getCode(char Symb)
    {
        return EXTENDED_SYMBOLS.indexOf(Symb);
    }

    public static char getSymbol(int code)
    {
        try {
            return EXTENDED_SYMBOLS.charAt(code);
        } catch (Exception ex) {
            return '?';
        }
    }

    public static final char[][] matchMap;
    static {
        Character simpleSymbols[] = {'A', 'C', 'G', 'T'};
        for (Character s : simpleSymbols) {
            putInMapping(s, s);
        }
        putInMapping('R', 'A', 'G');
        putInMapping('Y', 'C', 'T');
        putInMapping('M', 'A', 'C');
        putInMapping('K', 'G', 'T');
        putInMapping('W', 'A', 'T');
        putInMapping('S', 'C', 'G');
        putInMapping('B', 'C', 'G', 'T');
        putInMapping('D', 'A', 'G', 'T');
        putInMapping('H', 'A', 'C', 'T');
        putInMapping('V', 'A', 'C', 'G');
        putInMapping('N', 'A', 'C', 'G', 'T');
        putInMapping('_', '_');

        Set<Character> extSymbs = IUPACmapping.keySet();
        for (Character s : extSymbs) {
            HashSet<Character> set = IUPACmapping.get(s);
            IUPACreverseMapping.put(set, s);
            HashSet<Character> set2 = new HashSet<Character>();
            for (Character c : set) {
            	set2.add(Character.toLowerCase(c));
            }
            IUPACreverseMapping.put(set2, s);
        }
        matchMap = new char[EXTENDED_SYMBOLS.length()][EXTENDED_SYMBOLS.length()];

        char matchType = TRANSVERSION;
        for (int i = 0; i < EXTENDED_SYMBOLS.length(); ++i) {
            matchMap[i][i] = MATCH;
            char si = EXTENDED_SYMBOLS.charAt(i);
            HashSet<Character> siSet = IUPACmapping.get(si);
            for (int j = i + 1; j < EXTENDED_SYMBOLS.length(); ++j) {
                char sj = EXTENDED_SYMBOLS.charAt(j);
                HashSet<Character> sjSet = IUPACmapping.get(sj);
                // -----------------------------
                matchType = TRANSVERSION;
                for (char ni : siSet) {
                    for (char nj : sjSet) {
                        if (ni == nj) {
                            matchType = MATCH;
                            break;
                        } else if (isTransitionMutation(ni, nj)) {
                            matchType = TRANSITION;
                        }
                    }
                    if (matchType == MATCH) {
                        break;
                    }
                }
                matchMap[i][j] = matchType;
                matchMap[j][i] = matchType;
                // -----------------------------
            }
        }
    }

    private static void putInMapping(Character symbol, Character... symbols)
    {
        HashSet<Character> set = new HashSet<Character>(symbols.length);
        for (Character s : symbols) {
            set.add(s);
        }
        IUPACmapping.put(symbol, set);
    }

    public static char getExtendedSymbol(char... cs)
    {
        HashSet<Character> set = new HashSet<Character>(cs.length);
        for (char s : cs) {
            set.add(s);
        }
        return IUPACreverseMapping.get(set);
    }

    public static Set<Character> getExtendedSymbolSet(char cs)
    {
        return IUPACmapping.get(cs);
    }

    public static boolean isSimpleSymbol(char s)
    {
        return s == 'A' || s == 'C' || s == 'G' || s == 'T' || s == 'U' || s == 'a' || s == 'c' || s == 'g' || s == 't';
    }

    public static boolean isExtendedSymbol(char s)
    {
        return !isSimpleSymbol(s) && IUPACmapping.get(s) != null;
    }

    public static boolean isMatch(char cA, char cB)
    {
    	cA = Character.toUpperCase(cA);
    	cB = Character.toUpperCase(cB);
        try {
            return matchMap[EXTENDED_SYMBOLS.indexOf(cA)][EXTENDED_SYMBOLS.indexOf(cB)] == MATCH;
        } catch (Exception ex) {
            return false;
        }
    }

    public static boolean isTransitionMutationExt(char cA, char cB)
    {
    	cA = Character.toUpperCase(cA);
    	cB = Character.toUpperCase(cB);
        try {
            return matchMap[EXTENDED_SYMBOLS.indexOf(cA)][EXTENDED_SYMBOLS.indexOf(cB)] == TRANSITION;
        } catch (Exception ex) {
            return false;
        }
    }

    public static char getMatchType(char cA, char cB)
    {
    	cA = Character.toUpperCase(cA);
    	cB = Character.toUpperCase(cB);
        try {
            return matchMap[EXTENDED_SYMBOLS.indexOf(cA)][EXTENDED_SYMBOLS.indexOf(cB)];
        } catch (Exception ex) {
            return TRANSVERSION;
        }
    }

    public static char getComplementary(char symbol)
    {
        //symbol = Character.toUpperCase(symbol);
        switch (symbol) {
            case 'A':
                return 'T';
            case 'C':
                return 'G';
            case 'G':
                return 'C';
            case 'T':
                return 'A';
                
            case 'a':
                return 't';
            case 'c':
                return 'g';
            case 'g':
                return 'c';
            case 't':
                return 'a';

            case 'R':
                return 'Y';
            case 'Y':
                return 'R';
            case 'M':
                return 'K';
            case 'K':
                return 'M';
            case 'W':
                return 'S';
            case 'S':
                return 'W';
            case 'B':
                return 'V';
            case 'V':
                return 'B';
            case 'D':
                return 'H';
            case 'H':
                return 'D';
            case 'N':
                return 'N';

            default:
                return symbol;

        }
    }

    public static String getComplementary(String sequence)
    {
        StringBuilder cseq = new StringBuilder(sequence.length());
        for (char c : sequence.toCharArray()) {
            cseq.append(getComplementary(c));
        }
        return cseq.toString();
    }

    public static String getReverseComplementary(String sequence)
    {
        StringBuilder cseq = new StringBuilder(sequence.length());
        for (char c : sequence.toCharArray()) {
            cseq.append(getComplementary(c));
        }
        return cseq.reverse().toString();
    }

    public static boolean isTransitionMutation(char a, char b)
    {
        if (a > b) {
            return isTransitionMutation(b, a);
        }
        a = Character.toUpperCase(a);
        b = Character.toUpperCase(b);
        return (a == 'A' && b == 'G' || a == 'C' && b == 'T');
    }

    public static int getPrevCodonStart(int position, int phase)
    {
        if (position < phase) {
            return phase;
        }
        return position - (position - phase) % Codon.CODON_LEN;
    }

    public static int getPrevCodonStart(int position, int phase, boolean acceptNegatives)
    {
        if (position < phase) {
            return phase - Codon.CODON_LEN;
        }
        return position - (position - phase) % Codon.CODON_LEN;
    }

    public static int getNextCodonStart(int position, int phase)
    {
        if ((position - phase) % Codon.CODON_LEN == 0) {
            return position;
        }
        return position + Codon.CODON_LEN - (position - phase) % Codon.CODON_LEN;
    }

    public static String dna2rna(String dna)
    {
        return dna.replaceAll("T", "U");
    }

    public static String rna2dna(String rna)
    {
        return rna.replaceAll("U", "T");
    }

    public static DBSequence dna2rna(DBSequence dna)
    {
        return new DBSequence(dna.getSeq().replaceAll("T", "U"), dna.getId());
    }

    public static DBSequence rna2dna(DBSequence rna)
    {
        return new DBSequence(rna.getSeq().replaceAll("U", "T"), rna.getId());
    }

    public static int getNucleotideID(char n) {
    	n = Character.toUpperCase(n);
    	switch (n) {
    	case 'A':
    		return 0;
    	case 'C':
    		return 1;
    	case 'G':
    		return 2;
    	case 'T':
    	case 'U':
    		return 3;
    	}
    	// this is for reading the codons in the proper order from the PAML output
//    	switch (n) {
//    	case 'T':
//    	case 'U':
//    		return 0;
//    	case 'C':
//    		return 1;
//    	case 'A':
//    		return 2;
//    	case 'G':
//    		return 3;
//    	}
    	return Integer.MIN_VALUE;
    }
}
