package sequoia.path.util;

import java.util.ArrayList;

public class RandSeqGenerator
{
    private final static int AA_COUNT = 21;

    private int avgLen = 100;

    private int seqNo = 30;

    private int var = 50;

    private static char[] AAs;

    private static double[] intervals;

    public static void loadAAintervals()
    {
        AAs = GeneticCodeMap.getInstance().getAAs();
        System.out.println(AAs.length);
        intervals = new double[AA_COUNT + 1];
        intervals[0] = 0;

        for (int j = 0; j < AAs.length; ++j) {
            intervals[j + 1] = intervals[j] + GeneticCodeMap.getInstance().aafreqs.get(AAs[j] + "").doubleValue();
        }
        intervals[AA_COUNT] = 1;
    }

    public RandSeqGenerator()
    {

    }

    public RandSeqGenerator(int len, int No)
    {
        avgLen = len;
        seqNo = No;
        var = 0;// len / 4;
    }

    public ArrayList<String> generate()
    {
        ArrayList<String> seqs = new ArrayList<String>(seqNo);
        for (int i = 0; i < seqNo; ++i) {
            seqs.add(generateSeq());
        }
        return seqs;
    }

    public String generateSeq()
    {
        int len = avgLen + (int) ((0.5 - Math.random()) * var);
        // System.out.println(len + " " + avgLen + " " + var);
        StringBuilder strb = new StringBuilder(len);
        for (int i = 0; i < len; ++i) {
            strb.append(generateSymbol());
        }
        return strb.toString();
    }
    
    public ArrayList<String> generate(int len)
    {
        ArrayList<String> seqs = new ArrayList<String>(seqNo);
        for (int i = 0; i < seqNo; ++i) {
            seqs.add(generateSeq(len));
        }
        return seqs;
    }

    public String generateSeq(int len)
    {
        StringBuilder strb = new StringBuilder(len);
        for (int i = 0; i < len; ++i) {
            strb.append(generateSymbol());
        }
        return strb.toString();
    }

    public char generateSymbol()
    {
        double chance = Math.random();
        return getSelChar(chance);
    }

    private static char getSelChar(double chance)
    {
        for (int i = 0; i < AAs.length; ++i) {
            if (intervals[i] <= chance && intervals[i + 1] >= chance) {
                return AAs[i];
            }
        }
        return Codon.ANY_AA;
    }
}
