package sequoia.path.util;

import java.util.ArrayList;
import java.util.Arrays;

public enum RandomSequenceBackTranslator {
	
	instance;
	
    public void backTranslate(String AAsequence) {
    	
    	int codonslens[] = new int[AAsequence.length()];
    	int i = 0;
    	
    	for (char aa: AAsequence.toCharArray()) {
    		codonslens[i++] = GeneticCodeMap.getInstance().getCodons(aa).size();	
    	}
    	
    	int crtidx[] = new int[AAsequence.length()];
 
    	Arrays.fill(crtidx, 0);
    	
    	do {
    		System.out.println(backTranslate(AAsequence, crtidx));
    	} while (inc(crtidx, codonslens) != -1);
    }
    
    private int inc (int what[], int bound[]) {
    	return inc(what, bound, what.length - 1);
    }
    private int inc (int what[], int bound[], int pos) {
    	if (pos == -1) {
    		return -1;
    	}
    	if (++(what[pos]) >= bound[pos]) {
    		what[pos] = 0;
    		if (inc(what, bound, pos-1) < 0) {
    			return -1;
    		}
    	}
    	return what[pos];
    }
    
    public String backTranslate(String AAsequence, int choice[]) {
    	StringBuilder str = new StringBuilder(AAsequence.length() * Codon.CODON_LEN);
    	int i = 0;
    	for (char aa: AAsequence.toCharArray()) {
    		str.append(GeneticCodeMap.getInstance().getCodons(aa).get(choice[i++]).getSeq());
    	}
    	return str.toString();
    }
    
    public String backTranslateRandom(String AAsequence) {
    	StringBuilder str = new StringBuilder(AAsequence.length() * Codon.CODON_LEN);
    	for (char aa: AAsequence.toCharArray()) {
    		ArrayList<Codon> codons = GeneticCodeMap.getInstance().getCodons(aa);
    		int choice = (int)(Math.random() * codons.size());
    		str.append(codons.get(choice).getSeq());
    		
    	}
    	return str.toString();
    }

    public static void main(String args[]) {
    	for (String arg : args) {
    		RandomSequenceBackTranslator.instance.backTranslate(arg);
    	}
    }
}